PROGRAM FUND26;

(*  Climate Framework for Uncertainty, Negotiation and Distribution       *)
(*  Version 2.60  -  11 September 2002                                    *)
(*  Richard S.J. Tol                                                      *)
(*  Institute for Environmental Studies, Vrije Universiteit, Amsterdam    *)

(*  The Climate FUND is an integrated climate-economy model to calculate  *)
(*  the socio-economic impacts of both climate change and greenhouse gas  *)
(*  emission reduction, and to weight these two against one another in    *)
(*  order to determine the optimal abatement strategy in several game     *)
(*  theoretic settings. FUND is a highly parameterised, scenario-driven,  *)
(*  closed-loop model of the economies of nine major world regions and    *)
(*  their interactions with the climate. The framework includes a range   *)
(*  of climate change impacts, including intangible damages and forced    *)
(*  migration, and a suite of emission reduction measures, including      *)
(*  interregional capital transfers. The development of the Climate FUND  *)
(*  is sponsored by the Dutch National Research Programme: Global Air     *)
(*  Pollution and Climate Change, the EU DG Research Fourth Framework     *)
(*  Programme, the National Science Foundation, and the Michael Otto      *)
(*  Foundation.                                                           *)

USES
    Crt, RP260, GenRand;

CONST
     scratch = true;                  (* start with no measures ? *)
     CBA = false;                     (* cost-benefit analysis ? *)
     CEA = false;                     (* cost-effectiveness analysis ? *)
     cooperate = false;               (* cooperate ? *)
     repeated = false;
     uncertain = false;               (* uncertainty ? *)
     sequent = true;                  (* sequential ? *)
     scengmt = false;                 (* scenario global mean temperature ? *)
     kantian = false;
     compens = false;

     consleak = 0.25;                 (* consumption leak *)
     perm = 0.5;                      (* share permanent emission reduction *)
     knowpar = 0.9;
     knowgpar = 0.1;
     compth = 100;                    (* compensation threshold *)

     itmax = 25;

     startper = 1;
     noper = 20;
     repper = 20;

     riskave = 1;
     ineqave = 0;

     ampl = 4;
     altruism = 0.00; (* altruism should be zero if cooperation *)

     runs = 0;

     bforst = 1.00;
     btaxst = 0.0001;

     CBaU = 1792.3352;
     CMax = 550;

TYPE
    measure = regarr;
    matmeas = ARRAY [1..NoReg,1..NOI] OF double;
    regper = ARRAY [1..NoReg,1..NoPer] OF double;
    boolmea = ARRAY [1..NoReg,1..NOI] OF boolean;

VAR
   boolmeas: boolmea;
   regstep, forstep, taxstep: double;
   c2000,c2025,c2050,c2075,c2100,c2125,c2150,c2175,c2200:double;
   toptper,globemm,globcum,globknow,minint: double;
   optper: word;
   globviol: boolean;
   emmrec, count: regarr;
   coop: regper;

   spbmbg, spbmsd, vslbg, vslsd, vyllbg, vyllsd, vmorbbg, vmorbsd, valincbg, valincsd, valbase: double;
   wrelbg, wrelsd, wrnlbg, wrnlsd, agelbg, agelsd, agnlbg, agnlsd, forel, fornl: double;
   heelbg, heelsd, henlbg, henlsd, ceelbg, ceelsd, cenlbg, cenlsd: double;
   dfelbg, dfelsd, dfnlbg, dfnlsd, smelbg, smelsd, smnlbg, smnlsd, malelbg, malelsd, malnlbg, malnlsd: double;
   vbinc, vbiaddbg, vbiaddsd, bioloss, biosens, bioshare, nospecies: double;
   dvbmbg, dvbmsd, drydelbg, drydelsd, incdens, wvbmbg, wvbmsd, wetdelbg, wetdelsd: double;
   immcstbg, immcstsd, entertimebg, entertimesd, emcstbg, emcstsd, leavedelbg, leavedelsd: double;
   pluselbg, pluselsd, cvlinbg, cvlinsd, rlinbg, rlinsd, maxcardvascbg, maxcardvascsd: double;
   addmp: double;

FUNCTION Log(x: double): double;
(* returns ln(x) *)

BEGIN
     IF (x > 1.0E-308) AND (x < 1.0E308) THEN
        log:= ln(x)
     ELSE IF x < 1.0E-308 THEN
          log:= ln(1.0E-308)
     ELSE
         log:= ln(1.0E38)
END;

FUNCTION Expo(x: double): double;
(* returns exp(x) *)

BEGIN
     IF (x < 709) AND (x > -11356) THEN
        expo:= exp(x)
     ELSE IF x < -11356 THEN
          expo:= exp(-11356)
     ELSE
         expo:= exp(709)
END;

FUNCTION Root(x: double): double;
(* returns sqrt(x) *)

BEGIN
     IF (x < 1.0E308) AND (x > 0.00000000001) THEN
        root:= sqrt(x)
     ELSE IF x < 0.00000000001 THEN
          root:= 0
     ELSE
         root:= 1.0E154
END;

FUNCTION Quad(x: double): double;
(* returns x^2 *)

BEGIN
     IF x < 1.0E154 THEN
        quad:= sqr(x)
     ELSE
         quad:= 1.0E308
END;

FUNCTION Pow(x,y: double): double;
(* returns x^y *)

BEGIN
     pow:= expo(y*log(x))
END;

FUNCTION Interact(M,N: double): double;
(* interaction radiative forcing methane and nitrous oxide (in ppbv) *)

VAR hulp: double;

BEGIN
     hulp:= 1 + pow(M*N,0.75)*2.01E-5 + pow(M*N,1.52)*M*5.31E-15;
     interact:= 0.47*log(hulp)
END;

PROCEDURE InitImpactPar;

(* CODE
   sp = species
   for = forestry
   ag = agriculture
   he = heating energy
   ce = cooling energy
   wr = water resources
   df = dengue fever
   mal = malaria
   sm = schistosomiasis
   ccp = cardiovascular cold 65 plus
   ccm = cardiovascular cold 65 minus
   chp = cardiovascular heat 65 plus
   chm = cardiovascular heat 65 minus
   rp = respiratory 65 plus
   rm = respiratory 65 minus

   bg = best guess
   sd = standard deviation
   bm = benchmark
   nl = non-linearity
   el = elasticity
   del = delay
   l = linear
   q = quadratic
*)

VAR i,j: word;

BEGIN
     valbase:= 24401;
     valincbg:= 30000;
     valincsd:= 10000;
  (* 20000 is benchmark income; 15965.86 is OECD income in 1990 *)
     spbmbg:= 50;
     spbmsd:= 50;
     dvbmbg:= 4;
     dvbmsd:= 2;
     drydelbg:= 0;
     drydelsd:= 0.1;
     incdens:= 635;
     (* 635 is income density in 1990 in OECD *)
     wvbmbg:= 5;
     wvbmsd:= 2.5;
     wetdelbg:= 0;
     wetdelsd:= 0.1;
     immcstbg:= 0.4;
     immcstsd:= 0.2;
     entertimebg:= 3;
     entertimesd:= 1;
     emcstbg:= 3;
     emcstsd:= 1.5;
     leavedelbg:= 0;
     leavedelsd:= 0.1;
     wrelbg:= 0.85;
     wrelsd:= 0.15;
     wrnlbg:= 1;
     wrnlsd:= 0.5;
     forel:= -0.31;
     fornl:= 1;
     heelbg:= 0.8;
     heelsd:= 0.2;
     henlbg:= 0.5;
     henlsd:= 0.5;
     ceelbg:= 0.8;
     ceelsd:= 0.2;
     cenlbg:= 1.5;
     cenlsd:= 0.5;
     dfelbg:= 1.0;
     dfelsd:= 0.5;
     dfnlbg:= 1.0;
     dfnlsd:= 0.5;
     smelbg:= 1.0;
     smelsd:= 0.5;
     smnlbg:= 1.0;
     smnlsd:= 0.5;
     malelbg:= 1.0;
     malelsd:= 0.5;
     malnlbg:= 1.0;
     malnlsd:= 0.5;
     vbinc:= 3100;
     vbiaddbg:= 0;
     vbiaddsd:= 10;
     agnlbg:= 2;
     agnlsd:= 0.5;
     agelbg:= 0.31;
     agelsd:= 0.15;
     pluselbg:= 0.25;
     pluselsd:= 0.08;
     cvlinbg:= 0.025901;
     cvlinsd:= 0.009574;
     rlinbg:= 0.001583;
     rlinsd:= 0.000457;
     maxcardvascbg:= 0.05;
     maxcardvascsd:= 0.02;
     bioloss:= biolossbg;
     biosens:= biosensbg;
     bioshare:= biosharebg;

     vslbg:= 200;
     vslsd:= 100;
     vyllbg:= 10;
     vyllsd:= 5;
     vmorbbg:= 0.8;
     vmorbsd:= 1.2;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              spbm:= spbmbg;
              valinc:= valincbg;
              valadj:= valbase/valinc/(1+valbase/valinc);
              vsl:= vslbg;
              vyll:= vyllbg;
              vmorb:= vmorbbg;
              wrbm:= wrbmbg;
              wrel:= wrelbg;
              wrnl:= wrnlbg;
              watechrate:= watechratebg;
              forbm:= forbmbg;
              hebm:= hebmbg;
              heel:= heelbg;
              henl:= henlbg;
              cebm:= cebmbg;
              ceel:= ceelbg;
              cenl:= cenlbg;
              dfbm:= dfbmbg;
              dfel:= dfelbg;
              dfnl:= dfnlbg;
              smbm:= smbmbg;
              smel:= smelbg;
              smnl:= smnlbg;
              malbm:= malbmbg;
              malel:= malelbg;
              malnl:= malnlbg;
              vbiadd:= vbiaddbg;
              agrbm:= agrbmbg;
              aglbm:= aglbmbg;
              agopt:= agoptbg;
              aglparl:= -2*agopt*aglbm/(10.24-6.4*agopt);
              aglparq:= aglbm/(10.24-6.4*agopt);
              agcbm:= agcbmbg/ln(2);
              agtime:= agtimebg;
              agdel:= 1-1/agtime;
              agnl:= agnlbg;
              agel:= agelbg;
              ccplbm:= ccplbmbg;
              ccpqbm:= ccpqbmbg;
              ccmlbm:= ccmlbmbg;
              ccmqbm:= ccmqbmbg;
              chplbm:= chplbmbg;
              chpqbm:= chpqbmbg;
              chmlbm:= chmlbmbg;
              chmqbm:= chmqbmbg;
              cvlin:= cvlinbg;
              plusel:= pluselbg;
              maxcardvasc:= maxcardvascbg;
              rlin:= rlinbg;
              rlbm:= rlbmbg;
              rqbm:= rqbmbg;
              wlbm:= wlbmbg;
              wvbm:= wvbmbg;
              wetdel:= wetdelbg;
              wmbm:= wmbmbg;
              dlbm:= dlbmbg;
              dvbm:= dvbmbg;
              drydel:= drydelbg;
              pc:= pcbg;
              emcst:= emcstbg;
              immcst:= immcstbg;
              leavedel:= leavedelbg;
              enterdel:= 1-1/entertimebg;
              FOR j:= 1 TO NoReg DO
                  immigrate[j]:= immigratebg[j]
         END
END;

PROCEDURE InitCoalition;

VAR i,j: word;

BEGIN
       coop[1,1]:= 0;   coop[1,2]:= 0;   coop[1,3]:= 0;   coop[1,4]:= 0;
       coop[1,5]:= 0;   coop[1,6]:= 0;   coop[1,7]:= 0;   coop[1,8]:= 0;
       coop[1,9]:= 0;  coop[1,10]:= 0;  coop[1,11]:= 0;  coop[1,12]:= 0;
      coop[1,13]:= 0;  coop[1,14]:= 0;  coop[1,15]:= 0;  coop[1,16]:= 0;
      coop[1,17]:= 0;  coop[1,18]:= 0;  coop[1,19]:= 0;  coop[1,20]:= 0;

       coop[2,1]:= 0;   coop[2,2]:= 0;   coop[2,3]:= 0;   coop[2,4]:= 0;
       coop[2,5]:= 1;   coop[2,6]:= 1;   coop[2,7]:= 1;   coop[2,8]:= 1;
       coop[2,9]:= 1;  coop[2,10]:= 1;  coop[2,11]:= 1;  coop[2,12]:= 1;
      coop[2,13]:= 1;  coop[2,14]:= 1;  coop[2,15]:= 1;  coop[2,16]:= 1;
      coop[2,17]:= 1;  coop[2,18]:= 1;  coop[2,19]:= 1;  coop[2,20]:= 1;

       coop[3,1]:= 0;   coop[3,2]:= 0;   coop[3,3]:= 0;   coop[3,4]:= 0;
       coop[3,5]:= 0;   coop[3,6]:= 0;   coop[3,7]:= 0;   coop[3,8]:= 0;
       coop[3,9]:= 0;  coop[3,10]:= 0;  coop[3,11]:= 0;  coop[3,12]:= 0;
      coop[3,13]:= 0;  coop[3,14]:= 0;  coop[3,15]:= 0;  coop[3,16]:= 0;
      coop[3,17]:= 0;  coop[3,18]:= 0;  coop[3,19]:= 0;  coop[3,20]:= 0;

       coop[4,1]:= 1;   coop[4,2]:= 1;   coop[4,3]:= 1;   coop[4,4]:= 1;
       coop[4,5]:= 1;   coop[4,6]:= 1;   coop[4,7]:= 1;   coop[4,8]:= 1;
       coop[4,9]:= 1;  coop[4,10]:= 1;  coop[4,11]:= 1;  coop[4,12]:= 1;
      coop[4,13]:= 1;  coop[4,14]:= 1;  coop[4,15]:= 1;  coop[4,16]:= 1;
      coop[4,17]:= 1;  coop[4,18]:= 1;  coop[4,19]:= 1;  coop[4,20]:= 1;

       coop[5,1]:= 0;   coop[5,2]:= 0;   coop[5,3]:= 0;   coop[5,4]:= 0;
       coop[5,5]:= 0;   coop[5,6]:= 0;   coop[5,7]:= 0;   coop[5,8]:= 0;
       coop[5,9]:= 0;  coop[5,10]:= 0;  coop[5,11]:= 0;  coop[5,12]:= 0;
      coop[5,13]:= 0;  coop[5,14]:= 0;  coop[5,15]:= 0;  coop[5,16]:= 0;
      coop[5,17]:= 0;  coop[5,18]:= 0;  coop[5,19]:= 0;  coop[5,20]:= 0;

       coop[6,1]:= 0;   coop[6,2]:= 0;   coop[6,3]:= 0;   coop[6,4]:= 0;
       coop[6,5]:= 0;   coop[6,6]:= 0;   coop[6,7]:= 0;   coop[6,8]:= 0;
       coop[6,9]:= 1;  coop[6,10]:= 1;  coop[6,11]:= 1;  coop[6,12]:= 1;
      coop[6,13]:= 1;  coop[6,14]:= 1;  coop[6,15]:= 1;  coop[6,16]:= 1;
      coop[6,17]:= 1;  coop[6,18]:= 1;  coop[6,19]:= 1;  coop[6,20]:= 1;

       coop[7,1]:= 0;   coop[7,2]:= 0;   coop[7,3]:= 0;   coop[7,4]:= 0;
       coop[7,5]:= 0;   coop[7,6]:= 0;   coop[7,7]:= 1;   coop[7,8]:= 1;
       coop[7,9]:= 1;  coop[7,10]:= 1;  coop[7,11]:= 1;  coop[7,12]:= 1;
      coop[7,13]:= 1;  coop[7,14]:= 1;  coop[7,15]:= 1;  coop[7,16]:= 1;
      coop[7,17]:= 1;  coop[7,18]:= 1;  coop[7,19]:= 1;  coop[7,20]:= 1;

       coop[8,1]:= 0;   coop[8,2]:= 0;   coop[8,3]:= 0;   coop[8,4]:= 0;
       coop[8,5]:= 0;   coop[8,6]:= 0;   coop[8,7]:= 0;   coop[8,8]:= 0;
       coop[8,9]:= 0;  coop[8,10]:= 0;  coop[8,11]:= 0;  coop[8,12]:= 0;
      coop[8,13]:= 0;  coop[8,14]:= 0;  coop[8,15]:= 0;  coop[8,16]:= 0;
      coop[8,17]:= 0;  coop[8,18]:= 0;  coop[8,19]:= 0;  coop[8,20]:= 0;

       coop[9,1]:= 1;   coop[9,2]:= 1;   coop[9,3]:= 1;   coop[9,4]:= 1;
       coop[9,5]:= 1;   coop[9,6]:= 1;   coop[9,7]:= 1;   coop[9,8]:= 1;
       coop[9,9]:= 1;  coop[9,10]:= 1;  coop[9,11]:= 1;  coop[9,12]:= 1;
      coop[9,13]:= 1;  coop[9,14]:= 1;  coop[9,15]:= 1;  coop[9,16]:= 1;
      coop[9,17]:= 1;  coop[9,18]:= 1;  coop[9,19]:= 1;  coop[9,20]:= 1;

      coop[10,1]:= 0;  coop[10,2]:= 0;  coop[10,3]:= 0;  coop[10,4]:= 0;
      coop[10,5]:= 0;  coop[10,6]:= 0;  coop[10,7]:= 0;  coop[10,8]:= 0;
      coop[10,9]:= 0; coop[10,10]:= 0; coop[10,11]:= 0; coop[10,12]:= 0;
     coop[10,13]:= 0; coop[10,14]:= 0; coop[10,15]:= 0; coop[10,16]:= 0;
     coop[10,17]:= 0; coop[10,18]:= 0; coop[10,19]:= 0; coop[10,20]:= 0;

      coop[11,1]:= 1;  coop[11,2]:= 1;  coop[11,3]:= 1;  coop[11,4]:= 1;
      coop[11,5]:= 1;  coop[11,6]:= 1;  coop[11,7]:= 1;  coop[11,8]:= 1;
      coop[11,9]:= 1; coop[11,10]:= 1; coop[11,11]:= 1; coop[11,12]:= 1;
     coop[11,13]:= 1; coop[11,14]:= 1; coop[11,15]:= 1; coop[11,16]:= 1;
     coop[11,17]:= 1; coop[11,18]:= 1; coop[11,19]:= 1; coop[11,20]:= 1;

      coop[12,1]:= 0;  coop[12,2]:= 0;  coop[12,3]:= 0;  coop[12,4]:= 0;
      coop[12,5]:= 0;  coop[12,6]:= 0;  coop[12,7]:= 0;  coop[12,8]:= 0;
      coop[12,9]:= 0; coop[12,10]:= 0; coop[12,11]:= 0; coop[12,12]:= 0;
     coop[12,13]:= 0; coop[12,14]:= 0; coop[12,15]:= 0; coop[12,16]:= 0;
     coop[12,17]:= 0; coop[12,18]:= 0; coop[12,19]:= 0; coop[12,20]:= 0;

      coop[13,1]:= 0;  coop[13,2]:= 0;  coop[13,3]:= 0;  coop[13,4]:= 0;
      coop[13,5]:= 0;  coop[13,6]:= 0;  coop[13,7]:= 0;  coop[13,8]:= 0;
      coop[13,9]:= 1; coop[13,10]:= 1; coop[13,11]:= 1; coop[13,12]:= 1;
     coop[13,13]:= 1; coop[13,14]:= 1; coop[13,15]:= 1; coop[13,16]:= 1;
     coop[13,17]:= 1; coop[13,18]:= 1; coop[13,19]:= 1; coop[13,20]:= 1;

      coop[14,1]:= 0;  coop[14,2]:= 0;  coop[14,3]:= 0;  coop[14,4]:= 0;
      coop[14,5]:= 0;  coop[14,6]:= 0;  coop[14,7]:= 1;  coop[14,8]:= 1;
      coop[14,9]:= 1; coop[14,10]:= 1; coop[14,11]:= 1; coop[14,12]:= 1;
     coop[14,13]:= 1; coop[14,14]:= 1; coop[14,15]:= 1; coop[14,16]:= 1;
     coop[14,17]:= 1; coop[14,18]:= 1; coop[14,19]:= 1; coop[14,20]:= 1;

      coop[15,1]:= 0;  coop[15,2]:= 0;  coop[15,3]:= 0;  coop[15,4]:= 0;
      coop[15,5]:= 0;  coop[15,6]:= 0;  coop[15,7]:= 0;  coop[15,8]:= 0;
      coop[15,9]:= 0; coop[15,10]:= 0; coop[15,11]:= 0; coop[15,12]:= 0;
     coop[15,13]:= 0; coop[15,14]:= 0; coop[15,15]:= 0; coop[15,16]:= 0;
     coop[15,17]:= 0; coop[15,18]:= 0; coop[15,19]:= 0; coop[15,20]:= 0;

      coop[16,1]:= 1;  coop[16,2]:= 1;  coop[16,3]:= 1;  coop[16,4]:= 1;
      coop[16,5]:= 1;  coop[16,6]:= 1;  coop[16,7]:= 1;  coop[16,8]:= 1;
      coop[16,9]:= 1; coop[16,10]:= 1; coop[16,11]:= 1; coop[16,12]:= 1;
     coop[16,13]:= 1; coop[16,14]:= 1; coop[16,15]:= 1; coop[16,16]:= 1;
     coop[16,17]:= 1; coop[16,18]:= 1; coop[16,19]:= 1; coop[16,20]:= 1;

     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO 20 DO
             coop[i,j]:= 0
END;

PROCEDURE Initialise;

VAR i, j: word;
    fil: text;

BEGIN
     addmp:= 1;
     optper:= startper;
     C2200:= CBaU;
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              oda:= 0;
              new(ypcgrowth);
              new(pgrowth);
              new(aeei);
              new(acei);
              new(forestemm)
         END;
     WITH Climate DO BEGIN
          decay[1]:= 1;
          decay[2]:= exp(-1/lifeco2);
          decay[3]:= exp(-1/lifeco3);
          decay[4]:= exp(-1/lifeco4);
          decay[5]:= exp(-1/lifeco5);
          frac[1]:= 0.13;
          frac[2]:= 0.20;
          frac[3]:= 0.32;
          frac[4]:= 0.25;
          frac[5]:= 0.10;
          ch4decay:= ch4dec;
          n2odecay:= n2odec;
          ch4sd:= 6.8346;
          n2osd:= 0.0059;
          new(addco2);
          new(addch4);
          new(addn2o);
(*          new(scentemp); *)
          new(ch4em);
          new(n2oem);
          new(acco2);
          new(acch4);
          new(acn2o)
     END;

     Economy[1].name:=  'USA';
     Economy[2].name:=  'CAN';
     Economy[3].name:=  'WEU';
     Economy[4].name:=  'JPK';
     Economy[5].name:=  'ANZ';
     Economy[6].name:=  'EEU';
     Economy[7].name:=  'FSU';
     Economy[8].name:=  'MDE';
     Economy[9].name:=  'CAM';
     Economy[10].name:= 'LAM';
     Economy[11].name:= 'SAS';
     Economy[12].name:= 'SEA';
     Economy[13].name:= 'CHI';
     Economy[14].name:= 'MAF';
     Economy[15].name:= 'SSA';
     Economy[16].name:= 'SIS';

     Economy[1].GDP90:=  6343.0;
     Economy[2].GDP90:=   532.5;
     Economy[3].GDP90:=  6091.8;
     Economy[4].GDP90:=  5100.7;
     Economy[5].GDP90:=   359.5;
     Economy[6].GDP90:=   330.7;
     Economy[7].GDP90:=   915.1;
     Economy[8].GDP90:=   423.1;
     Economy[9].GDP90:=   303.0;
     Economy[10].GDP90:= 1070.9;
     Economy[11].GDP90:=  351.5;
     Economy[12].GDP90:=  456.2;
     Economy[13].GDP90:=  511.4;
     Economy[14].GDP90:=  137.3;
     Economy[15].GDP90:=  274.1;
     Economy[16].GDP90:=   34.6;

     Economy[1].pop90:=   254.1;
     Economy[2].pop90:=    27.8;
     Economy[3].pop90:=   377.6;
     Economy[4].pop90:=   166.4;
     Economy[5].pop90:=    20.2;
     Economy[6].pop90:=   122.1;
     Economy[7].pop90:=   289.6;
     Economy[8].pop90:=   189.2;
     Economy[9].pop90:=   111.4;
     Economy[10].pop90:=  295.1;
     Economy[11].pop90:= 1131.7;
     Economy[12].pop90:=  444.1;
     Economy[13].pop90:= 1184.1;
     Economy[14].pop90:=  117.7;
     Economy[15].pop90:=  494.4;
     Economy[16].pop90:=   38.9;

     Economy[1].urbcorr:=   0.1163;
     Economy[2].urbcorr:=   0.0659;
     Economy[3].urbcorr:=   0.1578;
     Economy[4].urbcorr:=   0.1558;
     Economy[5].urbcorr:=  -0.0516;
     Economy[6].urbcorr:=   0.0051;
     Economy[7].urbcorr:=  -0.0121;
     Economy[8].urbcorr:=  -0.0391;
     Economy[9].urbcorr:=  -0.0892;
     Economy[10].urbcorr:= -0.1592;
     Economy[11].urbcorr:=  0.3132;
     Economy[12].urbcorr:=  0.8026;
     Economy[13].urbcorr:=  0.6088;
     Economy[14].urbcorr:=  0.0611;
     Economy[15].urbcorr:=  0.3879;
     Economy[16].urbcorr:=  0.0095;

     Economy[1].coastpd:=   1.04;
     Economy[2].coastpd:=   1.07;
     Economy[3].coastpd:=   4.48;
     Economy[4].coastpd:=   1.04;
     Economy[5].coastpd:=   1.27;
     Economy[6].coastpd:=   3.01;
     Economy[7].coastpd:=   0.75;
     Economy[8].coastpd:=   3.42;
     Economy[9].coastpd:=   0.69;
     Economy[10].coastpd:=  1.97;
     Economy[11].coastpd:=  2.31;
     Economy[12].coastpd:=  1.80;
     Economy[13].coastpd:=  4.46;
     Economy[14].coastpd:= 17.61;
     Economy[15].coastpd:=  9.69;
     Economy[16].coastpd:=  2.51;

     FOR i:= 1 TO NoReg DO
         Economy[i].ypc90:= Economy[i].GDP90/Economy[i].pop90*1000;

     Economy[1].plus90:=  0.1239;
     Economy[2].plus90:=  0.1122;
     Economy[3].plus90:=  0.1473;
     Economy[4].plus90:=  0.1490;
     Economy[5].plus90:=  0.1115;
     Economy[6].plus90:=  0.1034;
     Economy[7].plus90:=  0.0931;
     Economy[8].plus90:=  0.0358;
     Economy[9].plus90:=  0.0384;
     Economy[10].plus90:= 0.0486;
     Economy[11].plus90:= 0.0406;
     Economy[12].plus90:= 0.0400;
     Economy[13].plus90:= 0.0856;
     Economy[14].plus90:= 0.0373;
     Economy[15].plus90:= 0.0291;
     Economy[16].plus90:= 0.0604;

     Economy[1].cardvasc90:=  0.00350;
     Economy[2].cardvasc90:=  0.00350;
     Economy[3].cardvasc90:=  0.00436;
     Economy[4].cardvasc90:=  0.00263;
     Economy[5].cardvasc90:=  0.00263;
     Economy[6].cardvasc90:=  0.00481;
     Economy[7].cardvasc90:=  0.00874;
     Economy[8].cardvasc90:=  0.00198;
     Economy[9].cardvasc90:=  0.00182;
     Economy[10].cardvasc90:= 0.00182;
     Economy[11].cardvasc90:= 0.00281;
     Economy[12].cardvasc90:= 0.00203;
     Economy[13].cardvasc90:= 0.00205;
     Economy[14].cardvasc90:= 0.00237;
     Economy[15].cardvasc90:= 0.00149;
     Economy[16].cardvasc90:= 0.00182;

     Economy[1].resp90:=  0.00053;
     Economy[2].resp90:=  0.00053;
     Economy[3].resp90:=  0.00050;
     Economy[4].resp90:=  0.00037;
     Economy[5].resp90:=  0.00037;
     Economy[6].resp90:=  0.00030;
     Economy[7].resp90:=  0.00052;
     Economy[8].resp90:=  0.00019;
     Economy[9].resp90:=  0.00039;
     Economy[10].resp90:= 0.00039;
     Economy[11].resp90:= 0.00039;
     Economy[12].resp90:= 0.00046;
     Economy[13].resp90:= 0.00011;
     Economy[14].resp90:= 0.00037;
     Economy[15].resp90:= 0.00038;
     Economy[16].resp90:= 0.00039;


     Economy[1].trade[0]:= 0;
     Economy[1].trade[1]:= 0;
     Economy[1].trade[2]:= 0;
     Economy[1].trade[3]:= 0;
     Economy[1].trade[4]:= 0;
     Economy[1].trade[5]:= 0;
     Economy[2].trade[0]:= 0;
     Economy[2].trade[1]:= 0;
     Economy[2].trade[2]:= 0;
     Economy[2].trade[3]:= 0;
     Economy[2].trade[4]:= 0;
     Economy[2].trade[5]:= 0;
     Economy[3].trade[0]:= 0;
     Economy[3].trade[1]:= 0;
     Economy[3].trade[2]:= 0;
     Economy[3].trade[3]:= 0;
     Economy[3].trade[4]:= 0;
     Economy[3].trade[5]:= 0;
     Economy[4].trade[0]:= 0;
     Economy[4].trade[1]:= 0;
     Economy[4].trade[2]:= 0;
     Economy[4].trade[3]:= 0;
     Economy[4].trade[4]:= 0;
     Economy[4].trade[5]:= 0;
     Economy[5].trade[0]:= 0;
     Economy[5].trade[1]:= 0;
     Economy[5].trade[2]:= 0;
     Economy[5].trade[3]:= 0;
     Economy[5].trade[4]:= 0;
     Economy[5].trade[5]:= 0;
     Economy[6].trade[0]:= 0;
     Economy[6].trade[1]:= 4.62;      (* 2 BRM *)
     Economy[6].trade[2]:= 3.69;
     Economy[6].trade[3]:= 2.77;
     Economy[6].trade[4]:= 1.85;
     Economy[6].trade[5]:= 0.92;
     Economy[7].trade[0]:= 0;
     Economy[7].trade[1]:= 4.62;      (* 2 BRM *)
     Economy[7].trade[2]:= 3.69;
     Economy[7].trade[3]:= 2.77;
     Economy[7].trade[4]:= 1.85;
     Economy[7].trade[5]:= 0.92;
     Economy[8].trade[0]:= 0;
     Economy[8].trade[1]:= 2.63;           (* 10 BRM *)
     Economy[8].trade[2]:= 2.10;
     Economy[8].trade[3]:= 1.58;
     Economy[8].trade[4]:= 1.05;
     Economy[8].trade[5]:= 0.53;
     Economy[9].trade[0]:= 0;
     Economy[9].trade[1]:= 0.53;        (* 0.8 BRM *)
     Economy[9].trade[2]:= 0.43;
     Economy[9].trade[3]:= 0.32;
     Economy[9].trade[4]:= 0.21;
     Economy[9].trade[5]:= 0.11;
     Economy[11].trade[0]:= 0;
     Economy[11].trade[1]:= 0.46;     (* 0.5 BRM *)
     Economy[11].trade[2]:= 0.37;
     Economy[11].trade[3]:= 0.28;
     Economy[11].trade[4]:= 0.19;
     Economy[11].trade[5]:= 0.09;
     Economy[12].trade[0]:= 0;
     Economy[12].trade[1]:= 0.46;     (* 0.5 BRM *)
     Economy[12].trade[2]:= 0.37;
     Economy[12].trade[3]:= 0.28;
     Economy[12].trade[4]:= 0.19;
     Economy[12].trade[5]:= 0.09;
     Economy[13].trade[0]:= 0;
     Economy[13].trade[1]:= 0.53;     (* 1.3 BRM *)
     Economy[13].trade[2]:= 0.42;
     Economy[13].trade[3]:= 0.32;
     Economy[13].trade[4]:= 0.21;
     Economy[13].trade[5]:= 0.11;
     Economy[14].trade[0]:= 0;
     Economy[14].trade[1]:= 2.63;           (* 10 BRM *)
     Economy[14].trade[2]:= 2.10;
     Economy[14].trade[3]:= 1.58;
     Economy[14].trade[4]:= 1.05;
     Economy[14].trade[5]:= 0.53;
     Economy[15].trade[0]:= 0;
     Economy[15].trade[1]:= 1.08;   (* 2.9 BRM *)
     Economy[15].trade[2]:= 0.86;
     Economy[15].trade[3]:= 0.65;
     Economy[15].trade[4]:= 0.43;
     Economy[15].trade[5]:= 0.22;
     Economy[16].trade[0]:= 0;
     Economy[16].trade[1]:= 0.53;        (* 0.8 BRM *)
     Economy[16].trade[2]:= 0.43;
     Economy[16].trade[3]:= 0.32;
     Economy[16].trade[4]:= 0.21;
     Economy[16].trade[5]:= 0.11;

     FOR i:= 1 TO NoReg DO
         FOR j:= 6 TO NoPer DO
             economy[i].trade[j]:= 0;

     ReadWater;
     ReadForestry;
     ReadHeating;
     ReadCooling;
     ReadDengue;
     ReadSchisto;
     ReadMalaria;
     ReadAgricult;
     ReadCardHeat;
     ReadCardCold;
     ReadResp;
     ReadYLL;
     ReadYLD;
     ReadWetland;
     ReadDryland;
     ReadProtect;
     ReadImmigrate;

     ReadIncome;
     ReadPopulation;
     ReadYpCGrowth;
     ReadAgrish;
     ReadPGrowth;
     ReadACEI;
     ReadAEEI;
     ReadForestEmm;
     ReadForParam;
     ReadDiscRate;
     ReadAddCO2;
     ReadAddCH4;
     ReadAddN2O;
     ReadCH4Em;
     ReadN2OEm;
(*     ReadScenGMT; *)
     ReadNCEmmRestr;
     ReadCEmmRestr;
     ReadScenUnc;
     WITH climate DO BEGIN
          temps:= (modet)/6.3/ln(2);
          seas:= modes;
          delaysea:= 1/lifesea;
          delaytemp:= 1/lifetemp
     END;
     bstsq:= quad(baset);
     dbstsq:= quad(dbst);
     bsssq:= quad(bases);
     dbsssq:= quad(dbss);
     forstep:= bforst;
     taxstep:= btaxst;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              ecgradd:= 0;
              pgadd:= 0;
              aeeiadd:= 0;
              aceiadd:= 0;
              foremadd:= 0;
              taxmp:= 1;
              formp:= 1;
         END;
     ReadAEEI;
END;

PROCEDURE ReInit;

VAR i, j: word;
fil: text;

BEGIN
     economy[1].emissint:= 1.584*0.273;
     economy[2].emissint:= 2.034*0.273;
     economy[3].emissint:= 1.188*0.273;
     economy[4].emissint:= 0.136*0.273;
     economy[5].emissint:= 0.706*0.273;
     economy[6].emissint:= 3.470*0.273;
     economy[7].emissint:= 3.959*0.273;
     economy[8].emissint:= 0.306*0.273;
     economy[9].emissint:= 0.841*0.273;
     economy[10].emissint:= 0.622*0.273;
     economy[11].emissint:= 0.734*0.273;
     economy[12].emissint:= 0.432*0.273;
     economy[13].emissint:= 2.233*0.273;
     economy[14].emissint:= 1.210*0.273;
     economy[15].emissint:= 0.704*0.273;
     economy[16].emissint:= 2.135*0.273;

     economy[1].cummemm:=  22967;
     economy[2].cummemm:=   1999;
     economy[3].cummemm:=  25854;
     economy[4].cummemm:=   1135;
     economy[5].cummemm:=    526;
     economy[6].cummemm:=   2199;
     economy[7].cummemm:=   3214;
     economy[8].cummemm:=     86;
     economy[9].cummemm:=    212;
     economy[10].cummemm:=   696;
     economy[11].cummemm:=   569;
     economy[12].cummemm:=   225;
     economy[13].cummemm:=   775;
     economy[14].cummemm:=    64;
     economy[15].cummemm:=   472;
     economy[16].cummemm:=     0;

     Economy[1].area:=   9207581;
     Economy[2].area:=   9872483;
     Economy[3].area:=   3705040;
     Economy[4].area:=    472934;
     Economy[5].area:=   9235211;
     Economy[6].area:=   2186128;
     Economy[7].area:=  21716915;
     Economy[8].area:=   6313671;
     Economy[9].area:=   2440575;
     Economy[10].area:= 17451759;
     Economy[11].area:=  5137567;
     Economy[12].area:=  4958546;
     Economy[13].area:= 10958008;
     Economy[14].area:=  3953026;
     Economy[15].area:= 24329621;
     Economy[16].area:=   332534;

     globcum:= 60994.3;
     globknow:= 1;
     nospecies:= nospecbase;

     InitImpactPar;
     totalpop:= 0;
     minint:= 1000;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              kant:= 0;
              income:= GDP50;
              population:= pop0;
              popdens:= population/area*1000000;
              IF riskave = 1 THEN
                 welf0:= log(1000*(1-consleak)*GDP50/pop0)
              ELSE
                  welf0:= pow(1000*(1-consleak)*GDP50/pop0,1-riskave)/(1-riskave);
              welfare:= welf0;
              YpC:= 1000*GDP50/pop0;                     (* ????????? *)
              YpC0:= YpC;
              urbpop:= (0.031*root(ypc)-0.011*root(popdens))/(1+0.031*root(ypc)-0.011*root(popdens));
              energint:= 1;                             (* confirmed *)
              energuse:= GDP50;
              ryg:= 0;
              reei:= 0;                                 (* confirmed *)
              rcei:= 0;                                 (* confirmed *)
              seei:= 0;
              scei:= 0;
              htax:= 0;
              know:= 1;
              emission:= emissint*energuse;
              emiss0:= emission;                        (* confirmed *)
              IF emiss0/GDP50 < minint THEN
                 minint:= emiss0/GDP50;
              totalpop:= totalpop + pop0;
              cdr:= 1;

              agrate:= agrbm*pow(0.005/dbst,agnl)/(1-agdel);
              wetmeas:= 0;
              dryland:= 0;
              drycost:= 0;
              wetcost:= 0;
              leave:= 0;
              cumleave:= 0;
              cumenter:= 0;
              entercost:= 0;
              leavecost:= 0;
              watech:= 1;

              eloss:= 0;
              sloss:= 0
     END;
     WITH climate DO BEGIN
          biodiv:= 1;
          ch4add:= 0;
          n2oadd:= 0;
          temp:= 0.20;
(*          scentemp^[0]:= temp; *)
          tsq:= 0;
          dt:= 0.004986;
          dtsq:= 0;
          sea:= 0;
          ssq:= 0;
          ds:= 0.001255;
          dssq:= 0;
          cbox[1]:= 283.53;
          cbox[2]:= 5.62;
          cbox[3]:= 6.29;
          cbox[4]:= 2.19;
          cbox[5]:= 0.15;
          acco2^[0]:= 0;
          acch4^[0]:= 1222;
          acn2o^[0]:= 296;
          FOR i:= 1 TO 5 DO
              acco2^[0]:= acco2^[0] + cbox[i];
          ch4n2o:= interact(ch4pre,n2opre)
     END
END;

(* TAX PROCEDURES *)

PROCEDURE SetTax1(i,j: word);
(* sets tax country I in decade J to zero *)

BEGIN
     economy[i].tax[j]:= 0;
END;

PROCEDURE SetTax2;
(* sets all taxes to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 0 TO CtrMax DO
             economy[i].tax[j]:= 0;
END;

PROCEDURE SetTax3(i: word; T: double);

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].tax[j]:= T
     ELSE
         economy[i].tax[optper]:= T;
END;

PROCEDURE SetTax4(T: measure);

VAR i,j: word;

BEGIN
     IF NOT sequent THEN
        FOR i:= 1 TO NoReg DO
            FOR j:= 0 TO CtrMax DO
                economy[i].tax[j]:= T[i]
     ELSE
         FOR i:= 1 TO NoReg DO
             economy[i].tax[optper]:= T[i]
END;

PROCEDURE SetTax5(i,j: word; T: double);

BEGIN
     economy[i].tax[j]:= T
END;

PROCEDURE ObjTax(i: word);
(* changes taxes and influences for economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].tax[j]:= economy[i].tax[j] + taxstep
     ELSE BEGIN
         economy[i].tax[optper]:= economy[i].tax[optper] + taxstep;
         (*write(economy[i].tax[optper]:6:4,' ',economy[i].income:7:0,' ')*)
     END
END;

PROCEDURE TaxObj(i: word);
(* changes taxes and influences one taxstep back for economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].tax[j]:= economy[i].tax[j] - taxstep
     ELSE
         economy[i].tax[optper]:= economy[i].tax[optper] - taxstep
END;

(* FORESTRY PROCEDURES *)

PROCEDURE SetInflFor(VAR country: regrec; t: word);
(* determines the influence of COUNTRY.FORESTRY on the economy of COUNTRY *)

VAR fca, fcd, fa, fd, th1, th2: double;

BEGIN
     WITH country DO BEGIN
          IF forestry[optper] > 0 THEN BEGIN
             th1:= forconst - forlinear + root(quad(forconst-forlinear)-4*forconst*forsquare);
             th1:= th1/2/forsquare;
             th2:= (3*forconst/2 - forlinear)/2/forsquare;
             IF forestry[optper] < (th1 + th2) THEN BEGIN
                fd:= forestry[optper];
                fa:= 0
             END
             ELSE IF forestry[optper] < (2*th1 + th2) THEN BEGIN
                  fd:= th1 + th2;
                  fa:= forestry[optper] - fd
             END
             ELSE BEGIN
                  fa:= th1;
                  fd:= forestry[optper] - fa
             END;
             IF fd > forestemm^[t] THEN BEGIN    (* !!! *)
                fd:= forestemm^[t];
                fa:= forestry[optper] - fd
             END;
             IF fd > 0 THEN BEGIN
                fcd:= 2*(forconst + forlinear*fd + forsquare*quad(fd))/3;
                IF fcd < forconst*fd THEN
                   fcd:= 2*forconst*fd/3
             END
             ELSE
                 fcd:= 0;
             IF fa > 0 THEN BEGIN
                fca:= forconst + forlinear*fa + forsquare*quad(fa);
                IF fca < forconst*fa THEN
                   fca:= forconst*fa
             END
             ELSE
                 fca:= 0;
             forcost:= fca + fcd
          END
          ELSE
              forcost:= 0
     END
END;

PROCEDURE SetFor1(i,j: word);
(* sets forestry in country I in decade J to zero *)

BEGIN
     economy[i].forestry[j]:= 0;
END;

PROCEDURE SetFor2;
(* sets all forestry to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO CtrMax DO
             economy[i].forestry[j]:= 0;
END;

PROCEDURE SetFor3(i: word; T: double);

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].forestry[j]:= T
     ELSE
         economy[i].forestry[optper]:= T;
END;

PROCEDURE SetFor4(T: measure);

VAR i,j: word;

BEGIN
     IF NOT sequent THEN
        FOR i:= 1 TO NoReg DO
            FOR j:= 0 TO CtrMax DO
                economy[i].forestry[j]:= T[i]
     ELSE
         FOR i:= 1 TO NoReg DO
             economy[i].forestry[optper]:= T[i]
END;

PROCEDURE ObjFor(i: word);
(* changes forestry in economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].forestry[j]:= economy[i].forestry[j] + forstep
     ELSE
         economy[i].forestry[optper]:= economy[i].forestry[optper] + forstep
END;

PROCEDURE ForObj(i: word);
(* changes forestry one forstep back for economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].forestry[j]:= economy[i].forestry[j] - forstep
     ELSE
         economy[i].forestry[optper]:= economy[i].forestry[optper] - forstep
END;

(* SETTINGS *)

PROCEDURE SetMeasures1;
(* sets all measures to zero *)

BEGIN
     SetTax2;
     SetFor2
END;

PROCEDURE SetMeasures2;

VAR i,j: word;

BEGIN
     FOR i:= 1 TO NoPer DO
         FOR j:= 1 TO NoReg DO
             economy[j].tax[i]:= random/100
END;

PROCEDURE SetMeasures3(w: word);

VAR s: string;
    inp: text;
    i,j: word;

BEGIN
     str(w,s);
     assign(inp,sessionpathname+'seq'+s+'.out');
     reset(inp);
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].tax[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].forestry[i]);
         readln(inp)
     END;
     close(inp)
END;

PROCEDURE SetMeasures4;

VAR inp: text;
    i,j: word;

BEGIN
     assign(inp,sessionpathname+'cea.out');
     reset(inp);
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].tax[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].forestry[i]);
         readln(inp)
     END;
     close(inp)
END;

FUNCTION CheckStop(w: word): boolean;

VAR s: string;
    inp0,inp1: text;
    i,j: word;
    h0,h1: double;
    stop: boolean;

BEGIN
     str(w-1,s);
     assign(inp0,sessionpathname+'seq'+s+'.out');
     reset(inp0);
     str(w,s);
     assign(inp1,sessionpathname+'seq'+s+'.out');
     reset(inp1);
     stop:= true;
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO BEGIN
             read(inp0,h0);
             read(inp1,h1);
             stop:= stop AND (ABS(h1-h0) < 0.5*btaxst)
         END;
         readln(inp0);
         readln(inp1);
         FOR j:= 1 TO NoReg DO BEGIN
             read(inp0,h0);
             read(inp1,h1);
             stop:= stop AND (ABS(h1-h0) < 0.5*bforst)
         END;
         readln(inp0);
         readln(inp1)
     END;
     close(inp0);
     close(inp1);
     checkstop:= stop
END;

(* CALCULATING PROCEDURES *)

PROCEDURE Calculate(VAR country: regrec; t: word);

VAR i, j, decade: word;
    s, h, henter, excessloss, popold, ic, dc, x, noemer, teller, seef, time: double;
    ygrowth: double;
    incth: boolean;

BEGIN
     time:= t;
     WITH country, climate DO BEGIN
          IF t > 50 THEN BEGIN
             ygrowth:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*
                       (1+0.01*(ypcgrowth^[t-1]+ecgradd*(t-50)-0.6*malaria/population/1000))
                        - 1;
             population:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*population;
             energint:= (1 - 0.01*(aeei^[t] + aeeiadd*(t-50)) - reei)*energint;
             emissint:= (1 - 0.01*(acei^[t] + aceiadd*(t-50)) - rcei)*emissint;
             watech:= watech*(1-watechrate)
          END
          ELSE BEGIN
               ygrowth:= (1+0.01*pgrowth^[t-1])*(1+0.01*ypcgrowth^[t-1]) - 1;
               population:= (1+0.01*pgrowth^[t-1])*population;
               energint:= (1 - 0.01*aeei^[t] - reei)*energint;
               emissint:= (1 - 0.01*acei^[t] - rcei)*emissint
          END;
          income:= (1 + ygrowth - taxmp*ryg)*income - formp*forcost*income/GDP50;
          IF (t > 50) AND (t <= 60) THEN
             income:= income + oda;
          w1950:= welfare/welf0;
          popold:= population;
          IF population < 0.000000001 THEN
             YpC:= 0
          ELSE
              YpC:= 1000*income/population;
          agrish:= agrish0*pow(ypc/ypc90,-agel);
          energuse:= (1-seei)*energint*income;
          emission:= (1-scei)*emissint*energuse;
          IF income > 0 THEN BEGIN
             cummemm:= cummemm+emission;
             globcum:= globcum+emission
          END
          ELSE BEGIN
               globcum:= globcum-cummemm;
               cummemm:= 0
          END;
          IF (t mod 10 = 0) AND (t >= 40) THEN BEGIN
             decade:= trunc((t-40)/10);
             htax:= tax[decade];
             IF t=NYear THEN
                htax:= 0;
             taxpar:= (2.24 - 0.24*root(emission/income-minint))*(0.4+0.6*perm);
             ryg:= taxpar*quad(htax)/know/globknow;
             reei:= perm*0.5*htax;
             rcei:= perm*0.5*htax;
(*             IF NOT ((name = 'USA') OR (name = 'CAN') OR (name = 'WEU') OR (name = 'JPK') OR (name = 'ANZ')) THEN BEGIN
                noemer:= 0;
                teller:= 0;
                FOR i:= 1 TO OECD DO BEGIN
                    noemer:= noemer + economy[i].income;
                    teller:= teller + economy[i].ryg*economy[i].income
                END;
                IF noemer > 0.0000001 THEN
                   ryg:= ryg + trade[decade]*teller/noemer
             END
*)          END;
          seei:= 0.9*seei + (1-perm)*0.5*htax*1.7;
          scei:= 0.9*scei + (1-perm)*0.5*htax*1.7;
          know:= know*root(1+knowpar*htax);
          globknow:= globknow*root(1+knowgpar*htax);
          setinflfor(country,t);

          species:= ((1-bioshare)*spbm+bioshare*biodiv*spbm)/valbase*
                    ypc/valinc/(1+ypc/valinc)/valadj*ypc*population/1000*
                    dt/dbsta/(1+dt/dbsta);
          water:= wrbm*watech*GDP90*pow(income/GDP90,wrel)*pow(temp,wrnl);
          forests:= forbm*income*pow(ypc/ypc90,forel)*(0.5*pow(temp,fornl)+0.5*ln(acco2^[t-1]/co2pre)*2.2972);
          heating:= hebm*(1-seei)*GDP90*pow(temp,henl)*pow(ypc/ypc90,heel)*population/pop90;
          cooling:= cebm*(1-seei)*GDP90*pow(temp,cenl)*pow(ypc/ypc90,ceel)*population/pop90;
          IF t > 50 THEN BEGIN
             hebm:= hebm*(1-0.01*(aeei^[t]+aeeiadd*(t-50))-reei);
             cebm:= cebm*(1-0.01*(aeei^[t]+aeeiadd*(t-50))-reei);
          END;
          IF t > 50 THEN
             incth:= (ypc < vbinc + vbiadd*(t-50))
          ELSE
              incth:= (ypc < vbinc);
          IF incth THEN BEGIN
                  dengue:= dfbm*population*pow(temp,dfnl)*pow((vbinc-ypc)/(vbinc-ypc90),dfel);
                  schisto:= smbm*population*pow(temp,smnl)*pow((vbinc-ypc)/(vbinc-ypc90),smel);
                  malaria:= malbm*population*pow(temp,malnl)*pow((vbinc-ypc)/(vbinc-ypc90),malel)
             END
          ELSE BEGIN
               dengue:= 0;
               schisto:= 0;
               malaria:= 0
          END;
          agrate:= agrbm*pow(dt/dbst,agnl) + agdel*agrate;
          aglevel:= aglparl*temp + aglparq*temp*temp;
          agco2:= agcbm*ln(acco2^[t-1]/co2pre);
          agcost:= (agrate+aglevel+agco2)*agrish*income;
          IF agcost > agrish*income THEN
             agcost:= agrish*income;

          plus:= plus90*pow(ypc/ypc90,plusel);
          urbpop:= (0.031*root(ypc)-0.011*root(popdens))/(1+0.031*root(ypc)-0.011*root(popdens))
                   /(1+urbcorr/(1+0.001*pow(time-40,2)));
          basecardvasc:= cardvasc90 + cvlin*(plus-plus90);
          baseresp:= resp90 + rlin*(plus-plus90);

          cardheat:= (chplbm*plus + chmlbm*(1-plus))*temp +
                     (chpqbm*plus + chmqbm*(1-plus))*temp*temp;
          cardheat:= cardheat*urbpop*population*10;
          IF cardheat > 1000*maxcardvasc*basecardvasc*urbpop*population THEN
             cardheat:= 1000*maxcardvasc*basecardvasc*urbpop*population;
          IF cardheat < 0 THEN
             cardheat:= 0;

          resp:= rlbm*temp + rqbm*temp*temp;
          resp:= resp*urbpop*population*10;
          IF resp > 1000*maxcardvasc*baseresp*urbpop*population THEN
             resp:= 1000*maxcardvasc*baseresp*urbpop*population;
          IF resp < 0 THEN
             resp:= 0;

          cardcold:= (ccplbm*plus + ccmlbm*(1-plus))*temp +
                     (ccpqbm*plus + ccmqbm*(1-plus))*temp*temp;
          cardcold:= cardcold*population*10;
          IF cardcold < -1000*maxcardvasc*basecardvasc*population THEN
             cardcold:= -1000*maxcardvasc*basecardvasc*population;
          IF cardcold > 0 THEN
             cardcold:= 0;
          dead:= dengue + schisto + malaria + cardheat + cardcold + resp;
          yll:= d2ld*dengue+d2ls*schisto+d2lm*malaria+d2lc*cardheat+d2lc*cardcold+d2lr*resp;
          yld:= d2dd*dengue+d2ds*schisto+d2dm*malaria+d2dc*cardheat+d2dc*cardcold+d2dr*resp;
          daly:= yll+yld;
          deadcost:= vsl*ypc*dead/1000000000;
          (*deadcost:= vyll*ypc*yll/1000000000;*)
          morbcost:= vmorb*ypc*yld/1000000000;

          wetval:= wvbm/valbase*ypc/valinc/(1+ypc/valinc)/valadj*ypc;
          IF (wetmax > 0) THEN
             IF wettot > wetmax*(1-bioshare/(1+bioshare)) THEN
                wetval:= wetval*2
             ELSE
                 wetval:= wetval*((1-bioshare)+bioshare*wetmax/(wetmax-wettot));
          dryval:= dvbm*popdens*ypc/1000/incdens;
          npprotcost:= pc*ds*100*(1+discrate+ygrowth)/(discrate+ygrowth);
          IF npprotcost < 0 THEN
             npprotcost:= 0;
          npwetcost:= wetval*wmbm*ds/(discrate+ygrowth);
          IF npwetcost < 0 THEN
             npwetcost:= 0;
          npdrycost:= dryval*dlbm*ds*(1+ygrowth)*(discrate+ygrowth)/discrate/discrate;
          IF npdrycost < 0.00000001 THEN BEGIN
             npdrycost:= 0;
             protlev:= 0
          END
          ELSE
              protlev:= 1-0.5*(npprotcost+npwetcost)/npdrycost;
          IF protlev < 0 THEN
             protlev:= 0;
          wetland:= wlbm*sea;
          wetmeas:= wmbm*ds*protlev + wetmeas;
          wettot:= wetland + wetmeas;
          wetann:= (wlbm + wmbm*protlev)*ds;
          IF wettot > wetmax THEN BEGIN
             wettot:= wetmax;
             wetann:= 0
          END;
          wetcost:= wetval*wetann/1000 + wetdel*wetcost;
          dryland:= dlbm*ds*(1-protlev) + dryland;
          area:= area - dlbm*ds*(1-protlev);
          drycost:= dryval*dlbm*ds*(1-protlev)/1000 + drydel*drycost;
          protcost:= protlev*pc*ds/10;
          enter:= 0;
          FOR j:= 1 TO NoReg DO
              enter:= enter + immigrate[j]*economy[j].leave;
          leave:= coastpd*popdens*dlbm*ds*(1-protlev);
          cumleave:= leave + cumleave;
          cumenter:= enter + cumenter;
          leavecost:= emcst*ypc*leave/1000000000 + leavedel*leavecost;
          entercost:= immcst*ypc*enter/1000000000 + enterdel*entercost;

          eloss:= - water - forests - heating - cooling - agcost + drycost + protcost + entercost;
(*write(-water/eloss:6:4,' ',-forests/eloss:6:4,' ',-heating/eloss:6:4,' ',-cooling/eloss:6:4,' ');
write(-agcost/eloss:6:4,' ',drycost/eloss:6:4,' ',protcost/eloss:6:4,' ',entercost/eloss:6:4,' ');
readln;*)
          IF eloss > income THEN
             eloss:= income;
          sloss:= species + deadcost + morbcost + wetcost + leavecost;

          IF t >= 40 THEN
             population:= population + (enter - dead - leave)/1000000;
          popdens:= population/area*1000000;
          welfare:= (1-consleak)*income - eloss;
          IF t >= 40 THEN
             income:= income - consleak*eloss/10;
          IF (population < 0.00000001) OR ((sloss+kant) >= welfare) OR (welfare <= 0) THEN
             welfare:= -100000000
          ELSE BEGIN
              welfare:= 1000*(welfare-sloss-kant)/population;
              IF riskave = 1 THEN
                 welfare:= log(welfare)
              ELSE
                  welfare:= pow(welfare,1-riskave)/(1-riskave)
          END;
          IF income < 0.001*population THEN
             income:= 0.001*population;
          eloss:= eloss/income;
          sloss:= sloss/income;
          IF t = 40+10*optper THEN BEGIN
             cdr:= 1;
             hNPVW:= welfare
          END;
          IF t > 40+10*optper THEN BEGIN
             cdr:= (1+discrate)*cdr;
             IF hNPVW >= 1.0E-309 THEN
                hNPVW:= hNPVW + welfare/cdr
          END;
          totalpop:= totalpop + population
     END
END;

PROCEDURE Write1(VAR country: regrec; t: word; VAR els,sls,
                     ent,mal,sch,den,car,res,lea,inc,wel,pop,mc: text);

BEGIN
     WITH country, climate DO BEGIN
          write(ent,enter:10:4,' ');
          write(lea,leave:10:4,' ');
          write(mal,malaria:10:4,' ');
          write(sch,schisto:10:4,' ');
          write(den,dengue:10:4,' ');
          write(car,cardheat+cardcold:10:4,' ');
          write(res,resp:10:4,' ');
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:8:4,' ');
             write(sls,sloss*100:8:4,' ');
             IF t >= 40 THEN BEGIN
                write(mc,2*taxpar*tax[trunc((t-40)/10)]*income*1000/emission/know/globknow:8:2,' ');
write(mc,-taxpar*tax[trunc((t-40)/10)]*income*1000/know/globknow/emission/root(1+tax[trunc((t-40)/10)]):8:2,' ')
             END
             ELSE
                 write(mc,0,' ',0,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
               write(mc,0,' ')
          END;
          write(inc,income:6:0,' ');
          write(wel,emission:6:0,' ');
          write(pop,population:6:0,' ');
     END
END;

PROCEDURE WriteM(VAR country: regrec; t: word; VAR els,sls,inc,pop: text);

BEGIN
     WITH country, climate DO BEGIN
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:14:10,' ');
             write(sls,sloss*100:14:10,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
          END;
          write(inc,income:12:6,' ');
          write(pop,population:12:6,' ');
     END
END;

PROCEDURE CalcClimate(j: word);
(* calculates the climatic variables at time J *)

VAR mco2, emfforest: double;
    i: word;
    ht, hdt: double;

BEGIN
     WITH climate DO BEGIN
          mco2:= 0;
          FOR i:= 1 TO NoReg DO
              WITH economy[i] DO BEGIN
                   mco2:= mco2 + emission + forestemm^[j] - forestry[trunc(j/10)]
              END;
          IF j > 50 THEN BEGIN
             biodiv:= nospecbase/nospecies;
             FOR i:= 1 TO NoReg DO
                 mco2:= mco2 + economy[i].foremadd*(j-50)
          END;
          FOR i:= 1 TO 5 DO
              cbox[i]:= cbox[i]*decay[i] + 0.000471*frac[i]*(mco2 + addmp*addco2^[j]);
          acco2^[j]:= cbox[1];
          FOR i:= 2 TO 5 DO
              acco2^[j]:= acco2^[j] + cbox[i];
          IF j > 50 THEN BEGIN
             acch4^[j]:= acch4^[j-1] + 0.3597*(ch4em^[j] + ch4add*(j-50) + addmp*addch4^[j])
                         - ch4decay*(acch4^[j-1]-ch4pre);
             acn2o^[j]:= acn2o^[j-1] + 0.2079*(n2oem^[j] + n2oadd*(j-50) + addmp*addn2o^[j])
                         - n2odecay*(acn2o^[j-1]-n2opre);
          END
          ELSE BEGIN
               acch4^[j]:= acch4^[j-1] + 0.3597*(ch4em^[j] + addch4^[j])
                         - ch4decay*(acch4^[j-1]-ch4pre);
               acn2o^[j]:= acn2o^[j-1] + 0.2079*(n2oem^[j] + addn2o^[j])
                         - n2odecay*(acn2o^[j-1]-n2opre);
          END;
          rfCO2:= 6.3*log(acco2^[j]/co2pre);
          rfCH4:= 0.036*(sqrt(acCH4^[j])-sqrt(ch4pre))
                       - interact(acch4^[j],n2opre) + CH4N2O;
          rfN2O:= 0.14*(sqrt(acN2O^[j])-sqrt(N2Opre))
                       - interact(ch4pre,acn2o^[j]) + CH4N2O;

          radforc:= rfCO2 + rfCH4 + rfN2O;
          dt:= delaytemp*temps*radforc - delaytemp*temp;
          temp:= dt + temp;
(*          IF scengmt THEN BEGIN
             temp:= scentemp^[j];
             dt:= scentemp^[j] - scentemp^[j-1]
          END; *)
          tsq:= quad(temp);
          dtsq:= quad(dt);
          IF j > 50 THEN
             nospecies:= nospecies*(1-bioloss-biosens*dt*dt/dbsta/dbsta);
          ds:= delaysea*seas*temp - delaysea*sea;
          sea:= ds + sea;
          ssq:= quad(sea);
          dssq:= quad(ds);
          IF j = 40+10*optper THEN
             toptper:= temp
     END
END;

PROCEDURE DrawRandom;

VAR i, j: word;
    hulp, fake,immsum: double;

(*bioloss, bioshare, biosense, watech, agriculture*)

BEGIN
    WITH climate DO BEGIN
         GenNormal(0,1,ch4add,n2oadd);
         ch4add:= ch4add*ch4sd;
         n2oadd:= n2oadd*n2osd;

         GenGamma(8.1270,0.3508,temps); (*mode 2.5; var 1*)
(*         GenGamma(4.9218,0.6375,temps); (*mode 2.5; var 2*)
(*         GenGamma(4.27,1.16,temps);       (*mode 3.8; sd 2.4*)
         temps:= temps/6.3/ln(2);
(*         GenGamma(6.3584,0.0317,seas); (* mode 0.17; var 0.08^2 *)
(*         GenGamma(4.0083,0.0565,seas); (* mode 0.17; var 2*0.08^2 *)
(*         GenGamma(7.4729,0.0263,seas);      (* mode 0.17; sd 0.07 *)
         GenGamma(5.9959077,0.0612581,seas); (* mode 0.30604; var 0.15^2 *)
         GenTriang(8,16,1/ch4dec,hulp);
         ch4decay:= 1/hulp;
         GenTriang(100,170,1/n2odec,hulp);
         n2odecay:= 1/hulp;
         GenTriang(0.5*lifetemp,2*lifetemp,lifetemp,hulp);
         delaytemp:= 1/hulp;
         GenTriang(0.5*lifesea,2*lifesea,lifesea,hulp);
         delaysea:= 1/hulp;
         GenNormGTZ(lifeco2,0.25*lifeco2,hulp,fake);
         decay[2]:= exp(-1/hulp);
         GenNormGTZ(lifeco3,0.25*lifeco3,hulp,fake);
         decay[3]:= exp(-1/hulp);
         GenNormGTZ(lifeco4,0.25*lifeco4,hulp,fake);
         decay[4]:= exp(-1/hulp);
         GenNormGTZ(lifeco5,0.25*lifeco5,hulp,fake);
         decay[5]:= exp(-1/hulp);
    END;
    FOR i:= 1 TO NoReg DO
        WITH economy[i] DO BEGIN
             gennormal(0,1,pgadd,ecgradd);
             pgadd:= pgadd*pgsd;
             ecgradd:= ecgradd*ecgrsd;
             gennormal(0,1,aeeiadd,aceiadd);
             aeeiadd:= aeeiadd*aeeisd;
             aceiadd:= aceiadd*aceisd;
             gennormal(0,1,foremadd,fake);
             foremadd:= foremsd*foremadd;

             gennormGTZ(1,0,spbm,vsl);
             spbm:= spbmbg + spbmsd*spbm;
             vsl:= vslbg + vslsd*vsl;
             gennormGTZ(1,0,valinc,fake);
             valinc:= valincbg + valincsd*valinc;
             valadj:= valbase/valinc/(1+valbase/valinc);

             gennormGTZ(1,0.5,malel,malnl);
             gennormGTZ(1,0.5,dfel,dfnl);
             gennormGTZ(1,0.5,smel,smnl);
             gennormal(0,1,malbm,dfbm);
             gennormal(0,1,smbm,vbiadd);
             malel:= malelbg + malelsd*malel;
             malnl:= malnlbg + malnlsd*malnl;
             dfel:= dfelbg + dfelsd*dfel;
             dfnl:= dfnlbg + dfnlsd*dfnl;
             smel:= smelbg + smelsd*smel;
             smnl:= smnlbg + smnlsd*smnl;
             malbm:= malbmbg + malbmsd*malbm;
             dfbm:= dfbmbg + dfbmsd*dfbm;
             smbm:= smbmbg + smbmsd*smbm;
             vbiadd:= vbiaddbg + vbiaddsd*vbiadd;

             gennormal(0,1,chplbm,chmlbm);
             gennormal(0,1,chpqbm,chmqbm);
             gennormal(0,1,ccplbm,ccmlbm);
             gennormal(0,1,ccpqbm,ccmqbm);
             gennormal(0,1,rlbm,rqbm);
             chplbm:= ccplbmbg + chplbmsd*chplbm;
             chmlbm:= chmlbmbg + chmlbmsd*chmlbm;
             chpqbm:= chpqbmbg + chpqbmsd*chpqbm;
             chmqbm:= chmqbmbg + chmqbmsd*chmqbm;
             ccplbm:= ccplbmbg + ccplbmsd*ccplbm;
             ccmlbm:= ccmlbmbg + ccmlbmsd*ccmlbm;
             ccpqbm:= ccpqbmbg + ccpqbmsd*ccpqbm;
             ccmqbm:= ccmqbmbg + ccmqbmsd*ccpqbm;
             rlbm:= rlbmbg + rlbmsd*rlbm;
             rqbm:= rqbmbg + rqbmsd*rqbm;
             gennormGTZ(0,1,cvlin,rlin);
             gennormGTZ(0,1,plusel,maxcardvasc);
             cvlin:= cvlinbg + cvlinsd*cvlin;
             rlin:= rlinbg + rlinsd*rlin;
             plusel:= pluselbg + pluselsd*plusel;
             maxcardvasc:= maxcardvascbg + maxcardvascsd*maxcardvasc;

             gennormal(0,1,wrbm,forbm);
             gennormGTZ(0,1,wrel,wrnl);
             gennormGTZ(0,1,forel,fornl);
             gennormal(0,1,hebm,cebm);
             gennormGTZ(0,1,heel,henl);
             gennormGTZ(0,1,ceel,cenl);
             wrbm:= wrbmbg + wrbmsd*wrbm;
             forbm:= forbmbg + forbmsd*forbm;
             wrel:= wrelbg + wrelsd*wrel;
             wrnl:= wrnlbg + wrnlsd*wrnl;
             forel:= 0.31 + 0.2*forel;
             fornl:= 1 + 0.5*fornl;
             hebm:= hebmbg + hebmsd*hebm;
             cebm:= cebmbg + cebmsd*cebm;
             heel:= heelbg + heelsd*heel;
             henl:= henlbg + henlsd*henl;
             ceel:= ceelbg + ceelsd*ceel;
             cenl:= cenlbg + cenlsd*cenl;

             gennormGTZ(0,1,agrbm,agtime);
             gennormGTZ(0,1,agnl,agel);
             gennormal(0,1,aglbm,agopt);
             agrbm:= agrbmbg + agrbmsd*agrbm;
             agtime:= agtimebg + agtimesd*agtime;
             agdel:= 1 - 1/agtime;
             aglbm:= aglbmbg + aglbmsd*aglbm;
             agopt:= agoptbg + agoptsd*agopt;
             agnl:= agnlbg + agnlsd*agnl;
             agel:= agelbg + agelsd*agel;

             gennormal(0,1,wvbm,dvbm);
             wvbm:= wvbmbg + wvbmsd*wvbm;
             dvbm:= dvbmbg + dvbmsd*dvbm;

             gennormal(0,1,wlbm,wmbm);
             wlbm:= wlbmbg + wlbmsd*wlbm;
             wmbm:= wmbmbg + wmbmsd*wmbm;

             gennormal(0,1,pc,dlbm);
             pc:= pcbg + pcsd*pc;
             dlbm:= dlbmbg + dlbmsd*dlbm;

             genexponential(wetdelsd,wetdel);
             genexponential(drydelsd,drydel);
             genexponential(leavedelsd,leavedel);

             FOR j:= 1 TO NoReg DO
                 genexponential(immigratesd[j],immigrate[j]);
             gennormGTZ(0,1,immigrate[i],entertime);
             immigrate[i]:= immigratebg[i] + immigratesd[i]*immigrate[i];
             immsum:= 0;
             FOR j:= 1 TO NoReg DO
                 immsum:= immsum + immigrate[j];
             FOR j:= 1 TO NoReg DO
                 immigrate[j]:= immigrate[j]/immsum;

             entertime:= entertimebg + entertimesd*entertime;
             enterdel:= 1 - 1/entertime;

             gennormGTZ(0,1,emcst,immcst);
             emcst:= emcstbg + emcstsd*emcst;
             immcst:= immcstbg + immcstsd*immcst;

             gengamma(5.8284,0.2071,taxmp);
             gengamma(5.8284,0.2071,formp);
(*             gengamma(5.8284,0.2071,cregmp);  (* var = 0.25, mode = 1 *)
(*             gengamma(9.8990,0.1124,taxmp);
             gengamma(9.8990,0.1124,formp);
             gengamma(9.8990,0.1124,cregmp);   (* var = 0.125, mode = 1 *)
        END
END;

PROCEDURE RunEconomy;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, run : word;
    worldemm, oecdemm, ann1emm, maxkant, loss, totloss: double;

BEGIN
(*write('.');*)
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        FOR run:= 1 TO runs DO BEGIN
(*gotoxy(3,23);*)write(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                totalpop:= 0;
                IF i < 40 THEN
                   minint:= 1000;
                FOR j:= 1 TO NoReg DO BEGIN
                    Calculate(economy[j],i);
                    IF economy[j].emission/economy[j].income < minint THEN
                       minint:= economy[j].emission/economy[j].income
                END;
                IF i = 49+10*optper THEN
                   FOR j:= 1 TO NoReg DO BEGIN
                       emmrec[j]:= emmrec[j]*(run-1)/run + economy[j].emission/run;
                       IF economy[j].emission > ncemmres[j,optper] THEN
                          count[j]:= count[j]+1
                   END;
                CalcClimate(i)
            END;
            FOR i:= 1 TO NoReg DO
                economy[i].NPVW:= economy[i].hNPVW/run +
                                     (run-1)*economy[i].NPVW/run
        END
     END
     ELSE BEGIN
          ReInit;
          FOR i:= 1 TO NYear DO BEGIN
              totalpop:= 0;
              maxkant:= 0;
              totloss:= 0;
              IF i < 40 THEN
                 minint:= 1000;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  totloss:= totloss + loss*economy[j].income;
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  IF economy[j].emission/economy[j].income < minint THEN
                     minint:= economy[j].emission/economy[j].income
              END;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                       END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss) - totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              IF i = 49+10*optper THEN BEGIN
                 worldemm:= 0;
                 oecdemm:= 0;
                 FOR j:= 1 TO NoReg DO BEGIN
                     IF j < 4 THEN
                        oecdemm:= oecdemm + economy[j].emission;
                     worldemm:= worldemm + economy[j].emission (*+
                                economy[j].foremp*economy[j].forestemm^[i] -
                                economy[j].forestry[trunc(i/10)]*);
                     emmrec[j]:= economy[j].emission
                 END;
                 ann1emm:= oecdemm + economy[4].emission (*- economy[2].emission*);
                 worldemm:= worldemm (*- economy[5].emission - economy[6].emission - economy[9].emission*);
                 globviol:= (worldemm > cemmres[optper])
              END;
              CalcClimate(i)
          END;
          C2200:= climate.acco2^[NYear];
          FOR i:= 1 TO NoReg DO
(*              IF C2200 > CMax THEN
                 economy[i].NPVW:= economy[i].hNPVW*
                                   (1-0.5*(noper-optper)/noper*(C2200-CMax)*(C2200-CMax)/(CBaU-CMax)/(CBaU-CMax))
              ELSE *)
                  economy[i].NPVW:= economy[i].hNPVW
     END
END;

PROCEDURE WriteEMF;

VAR i, j, k, run : word;
    oecd,noecd,clim:text;
    TCEm, TCFE, TInc, TCCC, TERC: double;

BEGIN
     ReInit;
     assign(oecd,sessionpathname+'oecd.out');
     rewrite(oecd);
     assign(noecd,sessionpathname+'noecd.out');
     rewrite(noecd);
     assign(clim,sessionpathname+'climate.out');
     rewrite(clim);
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         IF i < 40 THEN
            minint:= 1000;
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             IF economy[j].emission/economy[j].income < minint THEN
                minint:= economy[j].emission/economy[j].income
         END;
         CalcClimate(i);
         IF (*(i mod 10 = 0)*) 0=0 THEN BEGIN
            TCEm:= 0;
            TCFE:= 0;
            TInc:= 0;
            TCCC:= 0;
            TERC:= 0;
            FOR j:= 1 TO 5 DO
                WITH economy[j] DO BEGIN
                     TCEm:= TCEm + emission;
                     TCFE:= TCFE + forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income;
                     TCCC:= TCCC + (eloss+sloss)*income;
                     TERC:= TERC + 0.01*ryg*income + formp*forcost*income/GDP50
                END;
            writeln(oecd,TCEm:12:6,' ',TCFE:12:6,' ',TInc:12:6,' ',TCCC:12:6,' ',TERC:12:6);
            TCEm:= 0;
            TCFE:= 0;
            TInc:= 0;
            TCCC:= 0;
            TERC:= 0;
            FOR j:= 6 TO NoReg DO
                WITH economy[j] DO BEGIN
                     TCEm:= TCEm + emission;
                     TCFE:= TCFE + forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income;
                     TCCC:= TCCC + (eloss+sloss)*income;
                     TERC:= TERC + 0.01*ryg*income + formp*forcost*income/GDP50
                END;
            writeln(noecd,TCEm:12:6,' ',TCFE:12:6,' ',TInc:12:6,' ',TCCC:12:6,' ',TERC:12:6);
            WITH Climate DO BEGIN
                 write(clim,i+1950:4,' ',acco2^[i]:10:6,' ',acch4^[i]:10:6,' ',acn2o^[i]:10:6,' ',temp:10:6,' ');
                 writeln(clim,sea:12:6)
            END
         END;
     END;
     close(oecd);
     close(noecd);
     close(clim)
END;

PROCEDURE WriteTol;

VAR i, j, k, run : word;
    econ,clim:text;
    TCEm, TCFE, TInc, TCCC, TERC: double;

BEGIN
     ReInit;
     assign(econ,sessionpathname+'tolecon.out');
     rewrite(econ);
     assign(clim,sessionpathname+'tolclim.out');
     rewrite(clim);
     FOR j:= 1 TO NoReg DO
         WITH economy[j] DO BEGIN
              TCFE:= forestemm^[0];
              TERC:= 0.01*ryg*income + forcost;
              IF (income > 0.0000001) AND (population > 0.0000001) THEN BEGIN
               write(econ,1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
                 write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
                 writeln(econ,TERC:12:6,' ',terc/income*100:12:6,' ',name)
              END
              ELSE BEGIN
               write(econ,1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
                   write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
                   writeln(econ,TERC:12:6,' ',0,' ',name)
              END
         END;
     WITH Climate DO BEGIN
          writeln(clim,1950:4,' ',acco2^[20]:12:6,' ',temp:12:6,' ',sea:8:6);
     END;
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         IF i < 40 THEN
            minint:= 1000;
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             IF economy[j].emission/economy[j].income < minint THEN
                minint:= economy[j].emission/economy[j].income
         END;
         CalcClimate(i);
         IF (i=40) OR (i=50) OR (i=75) OR (i=100) OR (i=150) OR (i=250) THEN BEGIN
            FOR j:= 1 TO NoReg DO
                WITH economy[j] DO BEGIN
                     TCFE:= forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income;
                     TERC:= 0.01*ryg*income + formp*forcost*income/GDP50;
IF income > 0.00000001 THEN BEGIN
write(econ,i+1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
writeln(econ,TERC:12:6,' ',terc/income*100:12:6,' ',name)
END
ELSE BEGIN
write(econ,i+1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
writeln(econ,TERC:12:6,' ',0,' ',name)
END
                END;
            WITH Climate DO
                 writeln(clim,i+1950:4,' ',acco2^[i]:12:6,' ',temp:12:6,' ',sea:12:6);
         END;
     END;
     close(econ);
     close(clim)
END;

PROCEDURE RunEconomy1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,eap,sap,ent,mal,sch,den,car,res,lea,inc,wel,pop,clim,mc,dam:text;
    wunc,cunc,tunc,punc,gunc,iunc,eunc,runc: text;
    totpop,totinc,totimp,totemm,totred: double;
    totdam, toty, loss, totloss, maxkant: double;

BEGIN
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        assign(wunc,sessionpathname+'welfunc.out');
        rewrite(wunc);
        assign(cunc,sessionpathname+'concunc.out');
        rewrite(cunc);
        assign(tunc,sessionpathname+'tempunc.out');
        rewrite(tunc);
        assign(punc,sessionpathname+'popunc.out');
        rewrite(punc);
        assign(gunc,sessionpathname+'incunc.out');
        rewrite(gunc);
        assign(iunc,sessionpathname+'impunc.out');
        rewrite(iunc);
        assign(eunc,sessionpathname+'emmunc.out');
        rewrite(eunc);
        assign(runc,sessionpathname+'redunc.out');
        rewrite(runc);
        FOR run:= 1 TO runs DO BEGIN
writeln(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                totalpop:= 0;
                IF i < 40 THEN
                   minint:= 1000;
                FOR j:= 1 TO NoReg DO BEGIN
                    Calculate(economy[j],i);
                    IF economy[j].emission/economy[j].income < minint THEN
                       minint:= economy[j].emission/economy[j].income
                END;
                CalcClimate(i);
                IF (i>=50) AND (i mod 25 = 0) THEN BEGIN
                   write(tunc,climate.temp:8:4,' ');
                   totpop:= 0;
                   totinc:= 0;
                   totimp:= 0;
                   totemm:= 0;
                   totred:= 0;
                   FOR k:= 1 TO NoReg DO BEGIN
                       totpop:= totpop + economy[k].population;
                       totinc:= totinc + economy[k].income;
                       totemm:= totemm + economy[k].emission;
                       totimp:= totimp + (economy[k].eloss+economy[k].sloss)*economy[k].income;
                       totred:= totred + economy[k].taxmp*economy[k].ryg*economy[k].income;
                   END;
                   write(punc,totpop:10:4,' ');
                   write(gunc,totinc:10:4,' ');
                   write(eunc,totemm:10:4,' ');
                   write(iunc,totimp/totinc*100:8:4,' ');
                   write(runc,totred/totinc*100:10:8,' ')
                END
            END;
            writeln(tunc);
            writeln(punc);
            writeln(gunc);
            writeln(eunc);
            writeln(iunc);
            writeln(runc);
            FOR i:= 1 TO NoReg DO
                WITH Economy[i] DO BEGIN
                     NPVW:= hNPVW/run + (run-1)*NPVW/run;
                     write(wunc,hNPVW:20:4,' ')
                END;
            writeln(wunc);
            WITH climate DO BEGIN
                 c2200:= acco2^[NYear]/run + (run-1)*c2200/run;
                 c2175:= acco2^[NYear-25]/run + (run-1)*c2175/run;
                 c2150:= acco2^[NYear-50]/run + (run-1)*c2150/run;
                 c2100:= acco2^[NYear-75]/run + (run-1)*c2125/run;
                 c2100:= acco2^[NYear-100]/run + (run-1)*c2100/run;
                 c2075:= acco2^[NYear-125]/run + (run-1)*c2075/run;
                 c2050:= acco2^[NYear-150]/run + (run-1)*c2050/run;
                 c2025:= acco2^[NYear-175]/run + (run-1)*c2025/run;
                 c2000:= acco2^[NYear-200]/run + (run-1)*c2000/run;
                 write(cunc,acco2^[NYear-200]:10:4,' ',acco2^[NYear-175]:10:4,' ');
                 write(cunc,acco2^[NYear-150]:10:4,' ',acco2^[NYear-125]:10:4,' ');
                 write(cunc,acco2^[NYear-100]:10:4,' ',acco2^[NYear-75]:10:4,' ');
                 write(cunc,acco2^[NYear-50]:10:4,' ',acco2^[NYear-25]:10:4,' ');
                 writeln(cunc,acco2^[NYear]:10:4)
            END
        END;
        close(wunc);
        close(cunc);
        close(tunc);
        close(punc);
        close(eunc);
        close(gunc);
        close(iunc);
        close(runc);
     END
     ELSE BEGIN
          ReInit;
          assign(els,sessionpathname+'eloss.out');
          rewrite(els);
          assign(sls,sessionpathname+'sloss.out');
          rewrite(sls);
          assign(ent,sessionpathname+'enter.out');
          rewrite(ent);
          assign(mal,sessionpathname+'malaria.out');
          rewrite(mal);
          assign(sch,sessionpathname+'schisto.out');
          rewrite(sch);
          assign(den,sessionpathname+'dengue.out');
          rewrite(den);
          assign(car,sessionpathname+'cardio.out');
          rewrite(car);
          assign(res,sessionpathname+'respir.out');
          rewrite(res);
          assign(lea,sessionpathname+'leave.out');
          rewrite(lea);
          assign(inc,sessionpathname+'income.out');
          rewrite(inc);
          assign(wel,sessionpathname+'emissions.out');
          rewrite(wel);
          assign(pop,sessionpathname+'population.out');
          rewrite(pop);
          assign(clim,sessionpathname+'climate.out');
          rewrite(clim);
          assign(mc,sessionpathname+'margcost.out');
          rewrite(mc);
          assign(dam,sessionpathname+'damage.out');
          rewrite(dam);
          FOR i:= 1 TO NYear DO BEGIN
              totalpop:= 0;
              write(els,i+1950,' ');
              write(sls,i+1950,' ');
              write(ent,i+1950,' ');
              write(mal,i+1950,' ');
              write(sch,i+1950,' ');
              write(den,i+1950,' ');
              write(car,i+1950,' ');
              write(res,i+1950,' ');
              write(lea,i+1950,' ');
              write(inc,i+1950,' ');
              write(wel,i+1950,' ');
              write(pop,i+1950,' ');
              write(mc,i+1950,' ');
              write(dam,i+1950,' ');
              totdam:= 0;
              toty:= 0;
              maxkant:= 0;
              IF i < 40 THEN
                 minint:= 1000;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  IF economy[j].emission/economy[j].income < minint THEN
                     minint:= economy[j].emission/economy[j].income;
                  Write1(economy[j],i,els,sls,ent,mal,sch,den,car,res,lea,inc,wel,pop,mc);
                  totdam:= totdam + (economy[j].eloss+economy[j].sloss)*economy[j].income;
                  toty:= toty + economy[j].income
              END;
              totloss:= totdam;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                     END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss)-totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              CalcClimate(i);
              WITH climate DO
                   writeln(clim,i+1950:4,' ',acco2^[i]:10:6,' ',acch4^[i]:10:6,' ',acn2o^[i]:10:6,' ',temp:10:6,' ',sea:8:6);
              writeln(els);
              writeln(sls);
              writeln(ent);
              writeln(mal);
              writeln(sch);
              writeln(den);
              writeln(res);
              writeln(car);
              writeln(lea);
              writeln(inc);
              writeln(wel);
              writeln(pop);
              writeln(mc);
              writeln(dam,climate.temp:8:6,' ',totdam:12:4,' ',toty:12:4,' ',totdam/toty:10:6);
          END;
          close(els);
          close(sls);
          close(ent);
          close(mal);
          close(sch);
          close(den);
          close(car);
          close(res);
          close(lea);
          close(inc);
          close(wel);
          close(pop);
          close(mc);
          close(clim);
          close(dam);
          (*WriteTol;*)
          FOR i:= 1 TO NoReg DO
              economy[i].NPVW:= economy[i].hNPVW
     END
END;

PROCEDURE MargRun1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,inc,pop:text;
    totpop,totinc,totimp,totemm,totred: double;
    hp,hy,hw: regarr;

BEGIN
     ReInit;
     assign(els,sessionpathname+'eloss.prn');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.prn');
     rewrite(sls);
     assign(inc,sessionpathname+'income.prn');
     rewrite(inc);
     assign(pop,sessionpathname+'population.prn');
     rewrite(pop);
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         CalcClimate(i);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
END;

PROCEDURE MargRun2;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,inc,pop:text;
    totpop,totinc,totimp,totemm,totred: double;
    hp,hy,hw: regarr;

BEGIN
     ReInit;
     assign(els,sessionpathname+'eloss.out');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.out');
     rewrite(sls);
     assign(inc,sessionpathname+'income.out');
     rewrite(inc);
     assign(pop,sessionpathname+'population.out');
     rewrite(pop);
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         CalcClimate(i);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
END;

PROCEDURE ShortRep(name: string);

VAR i: word;
    outf: text;

BEGIN
     assign(outf,sessionpathname+name);
     rewrite(outf);
     WriteLn(outf,'     tax   reg    forestry  region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,tax[optper]:8:4,' ',forestry[optper]:8:2,'  ',name);
     WriteLn(outf);
     WriteLn(outf,'            NPVW        p        y        w       v    region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,NPVW:16:4,' ',population/pop0:8:4,' ',income/GDP50:8:4,' ',w1950:8:4,' ',ypc/ypc0:8:4,' ',name);
     WriteLn(outf);
     WriteLn(outf,'  income 90   region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,GDP50:11:6,' ',name);
     WriteLn(outf);
     FOR i:= 28 TO NYear DO
         IF ((i+1970) MOD 25) = 0 THEN
            Writeln(outf,climate.acco2^[i]:12:6);
     close(outf)
END;

FUNCTION Objective(psnr: word): double;
(* calculates the sum total of the regional NPWs *)

VAR j: word;
    objw: double;

BEGIN
     IF ineqave = 1 THEN BEGIN
        objw:= 1;
        FOR j:= 1 TO NoReg DO
            objw:= objw*economy[j].npvw
     END
     ELSE BEGIN
          objw:= 0;
          FOR j:= 1 TO psnr DO
              objw:= objw + pow(economy[j].npvw,1-ineqave);
          objw:= objw/(1-ineqave)
     END;
     objective:= objw
END;

(* OPTIMISING PROCEDURES *)

PROCEDURE CoopCBAction(VAR T,F: measure);
(* finds the optimal emission restriction by taxing (OECD) or energy and
   carbon efficiency (non-OECD) and forestry measures under cooperation *)

VAR i, rc, mc: word;
    obj0, hobj, mobj: double;

BEGIN
     RunEconomy;
     obj0:= objective(noreg);
     mobj:= -10000000;
     FOR i:= 1 TO NoReg DO BEGIN
         objtax(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 1;
         END;
         taxobj(i);

(*         objfor(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
             mobj:= hobj;
             rc:= i;
             mc:= 3;
         END;
         forobj(i) *)
     END;
     WHILE mobj > obj0 DO BEGIN
 (*          IF mc=1 THEN *)
              objtax(rc);
(*           IF mc=3 THEN
              objfor(rc); *)
           RunEconomy;
           obj0:= objective(noreg);
           mobj:= -10000000;
           FOR i:= 1 TO NoReg DO BEGIN
               objtax(i);
               RunEconomy;
               hobj:= objective(noreg);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 1;
               END;
               taxobj(i);

(*               objfor(i);
               RunEconomy;
               hobj:= objective(noreg);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 3;
               END;
               forobj(i) *)
           END
     END;
     FOR i:= 1 TO NoReg DO BEGIN
         T[i]:= economy[i].tax[optper];
         F[i]:= 0 (*economy[i].forestry[optper]*)
     END
END;

PROCEDURE CoopCEAction(psnr: word; VAR T,F: measure);

VAR i, j, rc, mc: word;
    obj0, hobj, mobj: double;

BEGIN

(*economy[2].tax[optper]:= europe[optper];*)

     RunEconomy;
     obj0:= objective(psnr);
(*write(obj0:8:4,' ');*)
     mobj:= -10000000;

         (*objtax(1);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 1;
            mc:= 1;
         END;
         taxobj(1);*)

     FOR i:= 1 TO psnr DO BEGIN
(*IF emmrec[i] > 0.9*ncemmres[i,optper] THEN BEGIN*)
         objtax(i);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 1;
         END;
         taxobj(i);
(*END;*)
(*         objfor(i);
         RunEconomy;
         hobj:= objective;
         IF (hobj > mobj) THEN BEGIN
             mobj:= hobj;
             rc:= i;
             mc:= 3;
         END;
         forobj(i) *)
     END;

(*         objtax(7);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 7;
            mc:= 1;
         END;
         taxobj(7);
         objtax(8);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 8;
            mc:= 1;
         END;
         taxobj(8);*)


     WHILE (*(mobj > obj0) OR*) globviol DO BEGIN
           (*IF mc=1 THEN*)
              objtax(rc);
           (*IF mc=3 THEN
              objfor(rc);*)
           RunEconomy;
           obj0:= objective(psnr);
(*write(obj0:8:4,' ');*)
           mobj:= -10000000;

              (* objtax(1);
               RunEconomy;
               hobj:= objective(psnr);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= 1;
                  mc:= 1;
               END;
               taxobj(1);*)

           FOR i:= 1 TO psnr DO BEGIN
(*IF emmrec[i] > 0.9*ncemmres[i,optper] THEN BEGIN*)
               objtax(i);
               RunEconomy;
               hobj:= objective(psnr);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 1;
               END;
               taxobj(i);
(*END;*)
(*               objfor(i);
               RunEconomy;
               hobj:= objective;
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 3;
               END;
               forobj(i)*)
           END;
(*         objtax(7);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 7;
            mc:= 1;
         END;
         taxobj(7);
         objtax(8);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 8;
            mc:= 1;
         END;
         taxobj(8);*)


     END;
     FOR i:= 1 TO NoReg DO BEGIN
         T[i]:= economy[i].tax[optper];
         F[i]:= 0(*economy[i].forestry[optper]*)
     END
END;

PROCEDURE NonCoopCEAction(VAR T,F: measure);

VAR i, j: word;
    emm0, tobj, fobj, obj0: double;

BEGIN
     FOR i:= 1 TO NoReg DO BEGIN
write(economy[i].name,' ');
         RunEconomy;
(*writeln(ncemmres[i,optper]:8:4,' ',emmrec[i]:8:4,' ',economy[i].tax[optper]:6:4);*)
         (*obj0:= economy[i].npvw;
         emm0:= emmrec[i];
         objtax(i);
         count[i]:= 0;
         RunEconomy;
         tobj:= economy[i].npvw;
         taxobj(i);
(*         objfor(i);
         RunEconomy;
         fobj:= economy[i].npvw;
         forobj(i);   *)
         WHILE emmrec[i] > (*1.1*)ncemmres[i,optper] (*) (count[i] > 0.5*runs) (*OR (tobj > obj0) OR (fobj > obj0)*) DO BEGIN
              (* IF tobj > fobj THEN*)
                  objtax(i)
              (* ELSE
                   objfor(i)*);
(*writeln(ncemmres[i,optper]:8:4,' ',emmrec[i]:8:4,' ',economy[i].tax[optper]:6:4);*)
               RunEconomy;
               (*obj0:= economy[i].npvw;
               emm0:= emmrec[i];
               objtax(i);
               count[i]:= 0;
               RunEconomy;
               tobj:= economy[i].npvw;
               taxobj(i);
            (*   objfor(i);
               RunEconomy;
               fobj:= economy[i].npvw;
               forobj(i)  *)
         END;
         T[i]:= economy[i].tax[optper];
         F[i]:= 0 (*economy[i].forestry[optper]*)
     END
END;

PROCEDURE CoopCBA(VAR T,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     CoopCBAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     CoopCBAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     CoopCBAction(T,F)
END;

PROCEDURE CoopCEA(psnr: word; VAR T,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     CoopCEAction(psnr,T,F);
     FOR i:= 1 TO psnr DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     CoopCEAction(psnr,T,F);
     FOR i:= 1 TO psnr DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     CoopCEAction(psnr,T,F)
END;

PROCEDURE NonCoopCEA(VAR T,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     NonCoopCEAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     NonCoopCEAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     NonCoopCEAction(T,F)
END;

PROCEDURE NCOptimisation(W: integer; VAR T,F: double);
(* finds the optimal emission restriction by taxing and forestry under
   non-cooperation *)

VAR objt, objf, obj0: double;
    i: word;

BEGIN
write(' ',economy[w].name);

     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1-coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
        (*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
     taxobj(W);
     WHILE ((objt > obj0) AND (economy[w].tax[optper] < 0.05)(*OR (objf > obj0)*)) DO BEGIN
          (* IF objt > objf THEN BEGIN *)
              objtax(W);
         (*  END
           ELSE BEGIN
                objfor(W);
                RunEconomy
           END; *)
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
             obj0:= economy[w].npvw*(1-coop[w,optper]);
             FOR i:= 1 TO NoReg DO
                  obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           objtax(W);
           RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           objt:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
           taxobj(W)
     END;
     IF economy[w].tax[optper] > 0 THEN
        taxobj(w);
(*     IF economy[w].forestry[optper] > 0 THEN
        forobj(w); *)

     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper];
     taxobj(W);
     WHILE ((objt > obj0) AND (economy[w].tax[optper] < 0.05) (*OR (objf > obj0)*)) DO BEGIN
(*           IF objt > objf THEN BEGIN *)
              objtax(W);
(*           END
           ELSE BEGIN
                objfor(W);
                RunEconomy
           END;           *)
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           obj0:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           objtax(W);
           RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           objt:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               objt:= objt + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
           taxobj(W)
     END;
     IF economy[w].tax[optper] > 0 THEN
        taxobj(w);
(*     IF economy[w].forestry[optper] > 0 THEN
        forobj(w); *)

     taxstep:= btaxst;
     forstep:= bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
     taxobj(W);
     WHILE ((objt > obj0) AND (economy[w].tax[optper] < 0.05) (*OR (objf > obj0)*)) DO BEGIN
(*           IF objt > objf THEN BEGIN *)
              objtax(W);
(*           END
           ELSE BEGIN
                objfor(W);
                RunEconomy
           END; *)
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           obj0:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           objtax(W);
           RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           objt:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               objt:= objt + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
           taxobj(W)
     END;
     T:= economy[w].tax[optper];
     F:= 0 (*economy[w].forestry[optper]*)
END;

PROCEDURE Optimal(T,F: measure);

var i: word;

BEGIN
     SetTax4(T);
     SetFor4(F);
     RunEconomy1;
     ShortRep('summary.out')
END;

(* EXIT PROCEDURE *)

PROCEDURE Exit;

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              dispose(ypcgrowth);
              dispose(pgrowth);
              dispose(aeei);
              dispose(acei);
              dispose(forestemm)
         END;
     WITH Climate DO BEGIN
          dispose(addco2);
          dispose(addch4);
          dispose(addn2o);
          dispose(ch4em);
          dispose(n2oem);
          dispose(acco2);
          dispose(acch4);
          dispose(acn2o)
     END
END;

(* MAIN PROCEDURES *)

PROCEDURE RunNonCoopCBA;

VAR T,F: measure;
    w, cnt: integer;
    outs: text;
    stop: boolean;
    s: string;
    seet, seef: double;

BEGIN
     SetMeasures1;
     IF sequent THEN BEGIN
        assign(outs,sessionpathname+'seq0.out');
        rewrite(outs);
        FOR optper:= startper TO NoPer DO BEGIN
            write(optper:2,' ');
            FOR w:= 1 TO NoReg DO BEGIN
                NCOptimisation(w,T[w],F[w]);
                economy[w].tax[optper]:= 0
            END;
            FOR w:= 1 TO NoReg DO
                write(outs,T[w]:6:4,' ');
            writeln(outs);
            FOR w:= 1 TO NoReg DO
                write(outs,F[w]:6:2,' ');
            writeln(outs);
        END;
        close(outs);
        cnt:= 0;
        REPEAT
              inc(cnt);
              str(cnt,s);
              assign(outs,sessionpathname+'seq'+s+'.out');
              rewrite(outs);
              SetMeasures3(cnt-1);
              FOR optper:= StartPer TO NoPer DO BEGIN
                  write(optper:2,' ');
                  FOR w:= 1 TO NoReg DO
                      WITH economy[w] DO BEGIN
                           seet:= tax[optper];
                           seef:= forestry[optper];
                           tax[optper]:= 0;
                           forestry[optper]:= 0;
                           NCOptimisation(w,T[w],F[w]);
                           tax[optper]:= seet;
                           forestry[optper]:= seef;
                      END;
                  FOR w:= 1 TO NoReg DO
                      write(outs,((*0.9*economy[w].tax[optper]+0.1*)T[w]):6:4,' ');
                  writeln(outs);
                  FOR w:= 1 TO NoReg DO
                      write(outs,((*0.9*economy[w].forestry[optper]+0.1*)F[w]):6:2,' ');
                  writeln(outs);
              END;
              close(outs);
              stop:= checkstop(cnt)
        UNTIL (stop OR (cnt = itmax))
     END
     ELSE BEGIN
          optper:= 0;
          FOR w:= 1 TO NoReg DO
              NCOptimisation(w,T[w],F[w]);
          Optimal(T,F)
     END
END;

PROCEDURE RunNonCoopCEA;

VAR T,F: measure;
    i, cnt: word;
    s: string;
    ceaout: text;

BEGIN
     SetMeasures1;
     assign(ceaout,sessionpathname+'seq0.out');
     rewrite(ceaout);
     FOR optper:= StartPer TO NoPer DO BEGIN
         write(optper:2,' ');
         RunEconomy;
         NonCoopCEA(T,F);
         FOR i:= 1 TO NoReg DO
             write(ceaout,T[i]:6:4,' ');
         writeln(ceaout);
         FOR i:= 1 TO NoReg DO
             write(ceaout,F[i]:6:2,' ');
         writeln(ceaout)
     END;
     close(ceaout);
(*     cnt:= 0;
     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(ceaout,sessionpathname+'seq'+s+'.out');
           rewrite(ceaout);
           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO NoPer DO BEGIN
               write(optper:2,' ');
               RunEconomy;
               NonCoopCEA(T,F);
               FOR i:= 1 TO NoReg DO
                   write(ceaout,T[i]:6:4,' ');
               writeln(ceaout);
               FOR i:= 1 TO NoReg DO
                   write(ceaout,F[i]:6:2,' ');
               writeln(ceaout)
           END;
           close(ceaout);
     UNTIL checkstop(cnt)*)
END;

PROCEDURE RunCoopCBA;

VAR T,F,seet,seef: measure;
    i, cnt: word;
    outs: text;
    s: string;
    stop: boolean;

BEGIN
     IF Sequent THEN BEGIN
        assign(outs,sessionpathname+'seq0.out');
        rewrite(outs);
        FOR optper:= StartPer TO NoPer DO BEGIN
            SetMeasures1;
            write(optper:2,' ');
            RunEconomy;
            FOR i:= 1 TO NoReg DO
                economy[i].basewf:= economy[i].npvw;
            CoopCBA(T,F);
            FOR i:= 1 TO NoReg DO
                write(outs,T[i]:8:6,' ');
            writeln(outs);
            FOR i:= 1 TO NoReg DO
                write(outs,F[i]:6:2,' ');
            writeln(outs);
        END;
        close(outs);
        cnt:= 0;
        REPEAT
              inc(cnt);
              str(cnt,s);
              assign(outs,sessionpathname+'seq'+s+'.out');
              rewrite(outs);
              SetMeasures3(cnt-1);
              FOR optper:= StartPer TO NoPer DO BEGIN
                  write(optper:2,' ');
                  FOR i:= 1 TO NoReg DO BEGIN
                      seet[i]:= economy[i].tax[optper];
                      seef[i]:= economy[i].forestry[optper];
                      economy[i].tax[optper]:= 0;
                      economy[i].forestry[optper]:= 0
                  END;
                  RunEconomy;
                  FOR i:= 1 TO NoReg DO
                      economy[i].basewf:= economy[i].npvw;
                  CoopCBA(T,F);
                  FOR i:= 1 TO NoReg DO BEGIN
                      economy[i].tax[optper]:= seet[i];
                      economy[i].forestry[optper]:= seef[i]
                  END;
                  FOR i:= 1 TO NoReg DO
                      write(outs,(0.75*economy[i].tax[optper]+0.25*T[i]):8:6,' ');
                  writeln(outs);
                  FOR i:= 1 TO NoReg DO
                      write(outs,(0.75*economy[i].forestry[optper]+0.25*F[i]):6:2,' ');
                  writeln(outs);
              END;
              close(outs);
              stop:= checkstop(cnt)
        UNTIL (stop OR (cnt = itmax))
     END
     ELSE BEGIN
         RunEconomy;
         FOR i:= 1 TO NoReg DO
             economy[i].basewf:= economy[i].npvw;
         CoopCBA(T,F);
         Optimal(T,F)
     END
END;

PROCEDURE RunRepCBA;

VAR T,F,seet,seef: measure;
    i, w, cnt: word;
    outs: text;
    s: string;
    stop: boolean;
    seetd, seefd: double;


BEGIN
     assign(outs,sessionpathname+'seq0.out');
     rewrite(outs);

     FOR optper:= StartPer TO RepPer DO BEGIN
         SetMeasures1;
         write(optper:2,' ');
         RunEconomy;
         FOR i:= 1 TO NoReg DO
             economy[i].basewf:= economy[i].npvw;
         CoopCBA(T,F);
         FOR i:= 1 TO NoReg DO
             write(outs,T[i]:6:4,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,F[i]:6:2,' ');
         writeln(outs);
     END;

     SetMeasures1;
     FOR optper:= RepPer + 1 TO NoPer DO BEGIN
         write(optper:2,' ');
         FOR w:= 1 TO NoReg DO BEGIN
             NCOptimisation(w,T[w],F[w]);
             economy[w].tax[optper]:= 0
         END;
         FOR w:= 1 TO NoReg DO
             write(outs,T[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,F[w]:6:2,' ');
         writeln(outs);
     END;
     close(outs);
     cnt:= 0;

     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(outs,sessionpathname+'seq'+s+'.out');
           rewrite(outs);

           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO RepPer DO BEGIN
               write(optper:2,' ');
               FOR i:= 1 TO NoReg DO BEGIN
                   seet[i]:= economy[i].tax[optper];
                   seef[i]:= economy[i].forestry[optper];
                   economy[i].tax[optper]:= 0;
                   economy[i].forestry[optper]:= 0
               END;
               RunEconomy;
               FOR i:= 1 TO NoReg DO
                     economy[i].basewf:= economy[i].npvw;
               CoopCBA(T,F);
               FOR i:= 1 TO NoReg DO BEGIN
                   economy[i].tax[optper]:= seet[i];
                   economy[i].forestry[optper]:= seef[i]
               END;
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].tax[optper]+0.1*)T[i]:6:4,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].forestry[optper]+0.1*)F[i]:6:2,' ');
               writeln(outs);
           END;

           FOR optper:= RepPer+1 TO NoPer DO BEGIN
               write(optper:2,' ');
               FOR w:= 1 TO NoReg DO
                   WITH economy[w] DO BEGIN
                        seetd:= tax[optper];
                        seefd:= forestry[optper];
                        tax[optper]:= 0;
                        forestry[optper]:= 0;
                        NCOptimisation(w,T[w],F[w]);
                        tax[optper]:= seetd;
                        forestry[optper]:= seefd;
                   END;
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].tax[optper]+0.1*)T[w]):6:4,' ');
                writeln(outs);
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].forestry[optper]+0.1*)F[w]):6:2,' ');
                writeln(outs);
           END;
           close(outs);
           stop:= checkstop(cnt)
     UNTIL (stop OR (cnt = itmax))
END;

PROCEDURE RunCoopCEA;

VAR T,F: measure;
    i,psnr: word;
    ceaout: text;

BEGIN
     SetMeasures1;
(*     RunNonCoopCEA;*)
     assign(ceaout,sessionpathname+'cea.out');
     rewrite(ceaout);
     IF Sequent THEN BEGIN
        FOR optper:= StartPer TO NoPer DO BEGIN
            write(optper:2,' ');
            RunEconomy;
            (*IF optper < 5 THEN
               psnr:= 4
            ELSE*) psnr:= 7 (*NoReg*);
            FOR i:= 1 TO NoReg DO
                economy[i].basewf:= economy[i].npvw;
            CoopCEA(psnr,T,F);
            FOR i:= 1 TO NoReg DO
                write(ceaout,T[i]:6:4,' ');
            writeln(ceaout);
            FOR i:= 1 TO NoReg DO
                write(ceaout,F[i]:6:2,' ');
            writeln(ceaout)
        END
     END;
     close(ceaout)
END;

PROCEDURE CalcMargExt(VAR out: text);

VAR els0,els1,sls0,sls1,inc0,pop0: text;
    oldinc,oldpop,newinc,newpop,elsbs,elspt,slsbs,slspt,df0,df1,df3,disc,md: regarr;
    i,j: word;
    year,marg0,marg1,marg3,margreg0,margreg1,margreg3: double;
    margwld0,margwld1,margwld3,margeu0,margeu1,margeu3: double;
    wldpop, wldinc: double;

BEGIN
     assign(els0,sessionpathname+'eloss.prn');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.prn');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc0,sessionpathname+'income.prn');
     reset(inc0);
     assign(pop0,sessionpathname+'populati.prn');
     reset(pop0);
     read(inc0,year);
     read(pop0,year);
     read(els0,year);
     read(els1,year);
     read(sls0,year);
     read(sls1,year);
     FOR i:= 1 TO NoReg DO BEGIN
         read(inc0,oldinc[i]);
         read(pop0,oldpop[i]);
         read(els0,elsbs[i]);
         read(els1,elspt[i]);
         read(sls0,slsbs[i]);
         read(sls1,slspt[i]);
         df0[i]:= 1;
         df1[i]:= 1;
         df3[i]:= 1
     END;
     marg0:= 0;
     marg1:= 0;
     marg3:= 0;
     margreg0:= 0;
     margreg1:= 0;
     margreg3:= 0;
     margwld0:= 0;
     margwld1:= 0;
     margwld3:= 0;
     margeu0:= 0;
     margeu1:= 0;
     margeu3:= 0;
     readln(inc0);
     readln(pop0);
     readln(els0);
     readln(els1);
     readln(sls0);
     readln(sls1);
     FOR i:= 2 TO NYear DO BEGIN
         read(inc0,year);
         read(pop0,year);
         read(els0,year);
         read(els1,year);
         read(sls0,year);
         read(sls1,year);
         wldpop:= 0;
         wldinc:= 0;
         FOR j:= 1 TO NoReg DO BEGIN
             read(inc0,newinc[j]);
             read(pop0,newpop[j]);
             wldpop:= wldpop + newpop[j];
             wldinc:= wldinc + newinc[j];
             read(els0,elsbs[j]);
             read(els1,elspt[j]);
             read(sls0,slsbs[j]);
             read(sls1,slspt[j])
         END;
         readln(inc0);
         readln(pop0);
         readln(els0);
         readln(els1);
         readln(sls0);
         readln(sls1);
         FOR j:= 1 TO NoReg DO BEGIN
             IF (newpop[j] < 0.00001) OR (newinc[j] < 0.00001) THEN
                disc[j]:= 0
             ELSE
                 disc[j]:= (newinc[j]/newpop[j]-oldinc[j]/oldpop[j])/(oldinc[j]/oldpop[j]);
         END;
         IF (i > 49) AND (i < 150) THEN BEGIN
            FOR j:= 1 TO NoReg DO
                md[j]:= (elspt[j]+slspt[j]-elsbs[j]-slsbs[j])*newinc[j];
            marg0:= marg0 + md[2]*df0[2];
            marg1:= marg1 + md[2]*df1[2];
            marg3:= marg3 + md[2]*df3[2];
            FOR j:= 1 TO NoReg DO BEGIN
                margreg0:= margreg0 + md[j]*df0[j];
                margreg1:= margreg1 + md[j]*df1[j];
                margreg3:= margreg3 + md[j]*df3[j]
            END;
            margwld0:= margwld0 + md[1]*df0[1]*wldinc*newpop[1]/wldpop/newinc[1];
            margwld1:= margwld1 + md[1]*df1[1]*wldinc*newpop[1]/wldpop/newinc[1];
            margwld3:= margwld3 + md[1]*df3[1]*wldinc*newpop[1]/wldpop/newinc[1];
            margeu0:= margeu0 + md[1]*df0[1]*newinc[2]*newpop[1]/newpop[2]/newinc[1];
            margeu1:= margeu1 + md[1]*df1[1]*newinc[2]*newpop[1]/newpop[2]/newinc[1];
            margeu3:= margeu3 + md[1]*df3[1]*newinc[2]*newpop[1]/newpop[2]/newinc[1];
            margwld0:= margwld0 + md[2]*df0[2];
            margwld1:= margwld1 + md[2]*df1[2];
            margwld3:= margwld3 + md[2]*df3[2];
            margeu0:= margeu0 + md[2]*df0[2];
            margeu1:= margeu1 + md[2]*df1[2];
            margeu3:= margeu3 + md[2]*df3[2];
            FOR j:= 3 TO NoReg DO
                IF newinc[j] > 0.00001 THEN BEGIN
                   margwld0:= margwld0 + md[j]*df0[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld1:= margwld1 + md[j]*df1[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld3:= margwld3 + md[j]*df3[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margeu0:= margeu0 + md[j]*df0[j]*newinc[2]*newpop[j]/newpop[2]/newinc[j];
                   margeu1:= margeu1 + md[j]*df1[j]*newinc[2]*newpop[j]/newpop[2]/newinc[j];
                   margeu3:= margeu3 + md[j]*df3[j]*newinc[2]*newpop[j]/newpop[2]/newinc[j];
                END;
            FOR j:= 1 TO NoReg DO BEGIN
                df0[j]:= df0[j]/(1.00+disc[j]);
                df1[j]:= df1[j]/(1.01+disc[j]);
                df3[j]:= df3[j]/(1.03+disc[j])
            END
         END;
         FOR j:= 1 TO NoReg DO BEGIN
             oldinc[j]:= newinc[j];
             oldpop[j]:= newpop[j]
         END
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc0);
     close(pop0);
     write(out,marg0:12:6,' ',margreg0:12:6,' ',margwld0:12:6,' ',margeu0:12:6);
     write(out,marg1:12:6,' ',margreg1:12:6,' ',margwld1:12:6,' ',margeu1:12:6);
     writeln(out,marg3:12:6,' ',margreg3:12:6,' ',margwld3:12:6,' ',margeu3:12:6)
END;

PROCEDURE CalcMarg(VAR out, out2: text);

VAR els0,els1,sls0,sls1,inc0,pop0: text;
    oldinc,oldpop,newinc,newpop,elsbs,elspt,slsbs,slspt,df0,df1,df3,disc,md,npvmd: regarr;
    i,j: word;
    year,margreg0,margreg1,margreg3, margwld0,margwld1,margwld3: double;
    wldpop, wldinc: double;

BEGIN
     assign(els0,sessionpathname+'eloss.prn');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.prn');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc0,sessionpathname+'income.prn');
     reset(inc0);
     assign(pop0,sessionpathname+'populati.prn');
     reset(pop0);
     read(inc0,year);
     read(pop0,year);
     read(els0,year);
     read(els1,year);
     read(sls0,year);
     read(sls1,year);
     FOR i:= 1 TO NoReg DO BEGIN
         read(inc0,oldinc[i]);
         read(pop0,oldpop[i]);
         read(els0,elsbs[i]);
         read(els1,elspt[i]);
         read(sls0,slsbs[i]);
         read(sls1,slspt[i]);
         df0[i]:= 1;
         df1[i]:= 1;
         df3[i]:= 1;
         npvmd[i]:= 0
     END;
     margreg0:= 0;
     margreg1:= 0;
     margreg3:= 0;
     margwld0:= 0;
     margwld1:= 0;
     margwld3:= 0;
     readln(inc0);
     readln(pop0);
     readln(els0);
     readln(els1);
     readln(sls0);
     readln(sls1);
     FOR i:= 2 TO NYear DO BEGIN
         read(inc0,year);
         read(pop0,year);
         read(els0,year);
         read(els1,year);
         read(sls0,year);
         read(sls1,year);
         wldpop:= 0;
         wldinc:= 0;
         FOR j:= 1 TO NoReg DO BEGIN
             read(inc0,newinc[j]);
             read(pop0,newpop[j]);
             wldpop:= wldpop + newpop[j];
             wldinc:= wldinc + newinc[j];
             read(els0,elsbs[j]);
             read(els1,elspt[j]);
             read(sls0,slsbs[j]);
             read(sls1,slspt[j])
         END;
         readln(inc0);
         readln(pop0);
         readln(els0);
         readln(els1);
         readln(sls0);
         readln(sls1);
         FOR j:= 1 TO NoReg DO BEGIN
             IF (newpop[j] < 0.00001) OR (newinc[j] < 0.00001) THEN
                disc[j]:= 0
             ELSE
                 disc[j]:= (newinc[j]/newpop[j]-oldinc[j]/oldpop[j])/(oldinc[j]/oldpop[j]);
         END;
         IF (i > 49) (*AND (i < 150)*) THEN BEGIN
            FOR j:= 1 TO NoReg DO BEGIN
                md[j]:= (elspt[j]+slspt[j]-elsbs[j]-slsbs[j])*newinc[j];
                npvmd[j]:= npvmd[j] + md[j]*df1[j];
                margreg0:= margreg0 + md[j]*df0[j];
                margreg1:= margreg1 + md[j]*df1[j];
                margreg3:= margreg3 + md[j]*df3[j];
                IF newinc[j] > 0.00001 THEN BEGIN
                   margwld0:= margwld0 + md[j]*df0[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld1:= margwld1 + md[j]*df1[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld3:= margwld3 + md[j]*df3[j]*wldinc*newpop[j]/wldpop/newinc[j];
                END;
                df0[j]:= df0[j]/(1.00+disc[j]);
                df1[j]:= df1[j]/(1.01+disc[j]);
                df3[j]:= df3[j]/(1.03+disc[j])
            END
         END;
         FOR j:= 1 TO NoReg DO BEGIN
             oldinc[j]:= newinc[j];
             oldpop[j]:= newpop[j]
         END
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc0);
     close(pop0);
     write(out,margreg0:14:8,' ',margwld0:14:8,' ');
     write(out,margreg1:14:8,' ',margwld1:14:8,' ');
     writeln(out,margreg3:14:8,' ',margwld3:14:8);
     FOR i:= 1 TO NoReg DO
         write(out2,npvmd[i]:14:8,' ');
     writeln(out2)
END;

PROCEDURE CalcMargODA(r: word; VAR out: text);

VAR els0,els1,sls0,sls1,inc0,pop0: text;
    oldinc,oldpop,newinc,newpop,elsbs,elspt,slsbs,slspt: regarr;
    i,j: word;
    year,margreg0,margreg1,margreg3: double;
    df0, df1, df3, disc, md: double;

BEGIN
     assign(els0,sessionpathname+'eloss.prn');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.prn');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc0,sessionpathname+'income.prn');
     reset(inc0);
     assign(pop0,sessionpathname+'populati.prn');
     reset(pop0);
     read(inc0,year);
     read(pop0,year);
     read(els0,year);
     read(els1,year);
     read(sls0,year);
     read(sls1,year);
     FOR i:= 1 TO NoReg DO BEGIN
         read(inc0,oldinc[i]);
         read(pop0,oldpop[i]);
         read(els0,elsbs[i]);
         read(els1,elspt[i]);
         read(sls0,slsbs[i]);
         read(sls1,slspt[i]);
     END;
     df0:= 1;
     df1:= 1;
     df3:= 1;
     margreg0:= 0;
     margreg1:= 0;
     margreg3:= 0;
     readln(inc0);
     readln(pop0);
     readln(els0);
     readln(els1);
     readln(sls0);
     readln(sls1);
     FOR i:= 2 TO NYear DO BEGIN
         read(inc0,year);
         read(pop0,year);
         read(els0,year);
         read(els1,year);
         read(sls0,year);
         read(sls1,year);
         FOR j:= 1 TO NoReg DO BEGIN
             read(inc0,newinc[j]);
             read(pop0,newpop[j]);
             read(els0,elsbs[j]);
             read(els1,elspt[j]);
             read(sls0,slsbs[j]);
             read(sls1,slspt[j])
         END;
         readln(inc0);
         readln(pop0);
         readln(els0);
         readln(els1);
         readln(sls0);
         readln(sls1);
         IF (newpop[r] < 0.00001) OR (newinc[r] < 0.00001) THEN
            disc:= 0
         ELSE
             disc:= (newinc[r]/newpop[r]-oldinc[r]/oldpop[r])/(oldinc[r]/oldpop[r]);
         IF (i > 49) AND (i < 150) THEN BEGIN
            md:= (elspt[r]+slspt[r]-elsbs[r]-slsbs[r])*newinc[r];
            margreg0:= margreg0 + md*df0;
            margreg1:= margreg1 + md*df1;
            margreg3:= margreg3 + md*df3;
            df0:= df0/(1.00+disc);
            df1:= df1/(1.01+disc);
            df3:= df3/(1.03+disc)
         END;
         FOR j:= 1 TO NoReg DO BEGIN
             oldinc[j]:= newinc[j];
             oldpop[j]:= newpop[j]
         END
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc0);
     close(pop0);
     write(out,margreg0:14:8,' ');
     write(out,margreg1:14:8,' ');
     writeln(out,margreg3:14:8,' ');
END;

PROCEDURE Main;

VAR i: word;

BEGIN
     Initialise;
     ReInit;
     IF CBA THEN
        IF cooperate THEN
           RunCoopCBA
        ELSE IF repeated THEN
             RunRepCBA
        ELSE
            RunNonCoopCBA
     ELSE IF CEA THEN
          IF cooperate THEN
             RunCoopCEA
          ELSE
              RunNonCoopCEA
     ELSE BEGIN
          write(' calculating');
          IF Scratch THEN
             SetMeasures1
          ELSE
              SetMeasures4;
          RunEconomy1;
          ShortRep('summary.out')
     END;
     Exit
END;

PROCEDURE MargMain;

VAR i: word;
    out, out2: text;

BEGIN
     Initialise;
     ReInit;
     write(' calculating');
     IF scratch THEN
        SetMeasures1
     ELSE
         SetMeasures4;
     addmp:= 0;
     MargRun1;
     addmp:= 1;
     MargRun2;
     assign(out,sessionpathname+'margunc.out');
     rewrite(out);
     assign(out2,sessionpathname+'regmarg.out');
     rewrite(out2);
     CalcMarg(out,out2);
     FOR i:= 1 TO Runs DO BEGIN
         ReInit;
         DrawRandom;
         addmp:= 0;
         MargRun1;
         addmp:= 1;
         MargRun2;
         CalcMarg(out,out2)
     END;
     close(out);
     close(out2);
     Exit;
END;

PROCEDURE ODAMain;

VAR i,j: word;
    out: text;

BEGIN
     Initialise;
     ReInit;
     write(' calculating');
     IF scratch THEN
        SetMeasures1
     ELSE
         SetMeasures4;
     assign(out,sessionpathname+'odamarg.out');
     rewrite(out);
     FOR i:= 8 TO NoReg DO BEGIN
         FOR j:= 1 TO NoReg DO
             economy[j].oda:= 0;
         ReInit;
(*         DrawRandom; *)
         economy[i].oda:= 0;
         MargRun1;
         economy[i].oda:= 0.001;
         economy[1].oda:= -0.001/5;
         economy[2].oda:= -0.001/5;
         economy[3].oda:= -0.001/5;
         economy[4].oda:= -0.001/5;
         economy[5].oda:= -0.001/5;
         MargRun2;
         CalcMargODA(i,out)
     END;
     close(out);
     exit
END;

PROCEDURE AltMain;

VAR p,i,r: word;
    taks: double;
    outt, outw: text;

BEGIN
     Initialise;
     assign(outt,sessionpathname+'randtax.out');
     rewrite(outt);
     assign(outw,sessionpathname+'randwel.out');
     rewrite(outw);
     FOR p:= StartPer TO NoPer DO BEGIN
writeln(p:3);
         optper:= p;
         FOR i:= 1 TO 1000 DO BEGIN
             SetMeasures2;
             FOR r:= 1 TO NoReg DO BEGIN
                 taks:= random/100;
                 SetTax5(r,p,taks);
                 write(outt,taks:6:4,' ',taks*taks:10:8,' ')
             END;
             runeconomy;
             write(outt,toptper:10:6,' ',climate.temp:10:6);
             FOR r:= 1 TO NoReg DO
                 write(outw,economy[r].npvw:8:4,' ');
             writeln(outw);
             writeln(outt)
         END
     END;
     close(outw);
     close(outt)
END;

PROCEDURE AltMain2;

VAR i,r: word;
    outt: text;

BEGIN
     assign(outt,sessionpathname+'npv.out');
     rewrite(outt);
     Initialise;
     IF Scratch THEN
        SetMeasures1
     ELSE
         SetMeasures4;
     optper:= 1;
     objtax(3);objtax(12);objtax(14);objtax(15);objtax(3);objtax(14);objtax(15);objtax(14);objtax(15);
     runeconomy;
     FOR r:= 1 TO NoReg DO
         write(outt,economy[r].npvw:12:8,' ');
     writeln(outt);
     FOR i:= 1 TO NoReg DO BEGIN
         objtax(i);
         runeconomy;
         FOR r:= 1 TO NoReg DO
             write(outt,economy[r].npvw:12:8,' ');
         writeln(outt);
         taxobj(i)
     END;
     close(outt)
END;

BEGIN
     Main
END.